Imports Benday.Npr.Common.Repositories
Imports Benday.Npr.Common.DomainModel
Imports Benday.Npr.Common

Namespace Benday.Npr.Common
	Public NotInheritable Class XmlUtility
		Private Sub New()
		End Sub

		Public Shared Function GetAttributeValue(ByVal element As XElement, ByVal attributeName As String, ByVal defaultReturnValue As String) As String
			If element Is Nothing Then
				Return defaultReturnValue
			Else
				Dim value = element.Attribute(attributeName)
				If value Is Nothing Then
					Return defaultReturnValue
				Else
					Return value.Value
				End If
			End If
		End Function

		Public Shared Sub SetChildElement(ByVal parentElement As XElement, ByVal childElementName As String, ByVal childElementValue As Double)
			SetChildElement(parentElement, childElementName, childElementValue.ToString)
		End Sub

		Public Shared Sub SetChildElement(ByVal parentElement As XElement, ByVal childElementName As String, ByVal childElementValue As Date)
			SetChildElement(parentElement, childElementName, childElementValue.ToString)
		End Sub

		Public Shared Sub SetChildElement(ByVal parentElement As XElement, ByVal childElementName As String, ByVal childElementValue As String)
			If parentElement Is Nothing Then
				Throw New ArgumentNullException("parentElement", "parentElement is null.")
			End If
			If String.IsNullOrEmpty(childElementName) Then
				Throw New ArgumentException("childElementName is null or empty.", "childElementName")
			End If
			If String.IsNullOrEmpty(childElementValue) Then
				childElementValue = String.Empty
			End If

			parentElement.SetElementValue(childElementName, childElementValue)
		End Sub

		Public Shared Function GetChildElement(ByVal parentElement As XElement, ByVal childElementName As String) As XElement
			If parentElement Is Nothing Then
				Throw New ArgumentNullException("parentElement", "parentElement is null.")
			End If
			If String.IsNullOrEmpty(childElementName) Then
				Throw New ArgumentException("childElementName is null or empty.", "childElementName")
			End If

			Dim childElement = parentElement.Descendants(childElementName).FirstOrDefault
			Return childElement
		End Function

		Public Shared Function GetChildElementValue(ByVal parentElement As XElement, ByVal childElementName As String, ByVal defaultReturnValue As String) As String
			If parentElement Is Nothing Then
				Throw New ArgumentNullException("parentElement", "parentElement is null.")
			End If
			If String.IsNullOrEmpty(childElementName) Then
				Throw New ArgumentException("childElementName is null or empty.", "childElementName")
			End If

			Dim childElement As XElement = GetChildElement(parentElement, childElementName)

			If childElement Is Nothing Then
				Return defaultReturnValue
			Else
				Return childElement.Value
			End If
		End Function

		Public Shared Function GetChildElementValue(ByVal parentElement As XElement, ByVal childElementName As String, ByVal defaultReturnValue As Date) As Date
            If parentElement Is Nothing Then
                Throw New ArgumentNullException("parentElement", "parentElement is null.")
            End If
			If String.IsNullOrEmpty(childElementName) Then
				Throw New ArgumentException("childElementName is null or empty.", "childElementName")
			End If

			Dim childElement As XElement = GetChildElement(parentElement, childElementName)

			If childElement Is Nothing Then
				Return defaultReturnValue
			Else
				Dim returnValue As Date

				If Date.TryParse(childElement.Value, returnValue) = True Then
					Return returnValue
				Else
					Return defaultReturnValue
				End If
			End If
		End Function

		Public Shared Function GetChildElementValue(ByVal parentElement As XElement, ByVal childElementName As String, ByVal defaultReturnValue As Integer) As Integer
			If parentElement Is Nothing Then
				Throw New ArgumentNullException("parentElement", "parentElement is null.")
			End If
			If String.IsNullOrEmpty(childElementName) Then
				Throw New ArgumentException("childElementName is null or empty.", "childElementName")
			End If

			Dim childElement As XElement = GetChildElement(parentElement, childElementName)

			If childElement Is Nothing Then
				Return defaultReturnValue
			Else
				Dim returnValue As Integer

				If Int32.TryParse(childElement.Value, returnValue) = True Then
					Return returnValue
				Else
					Return defaultReturnValue
				End If
			End If
		End Function

		Public Shared Function GetChildElementValue(ByVal parentElement As XElement, ByVal childElementName As String, ByVal childAttributeName As String, ByVal childAttributeValue As String, ByVal defaultReturnValue As String) As String
			If parentElement Is Nothing Then
				Throw New ArgumentNullException("parentElement", "parentElement is null.")
			End If
			If String.IsNullOrEmpty(childElementName) Then
				Throw New ArgumentException("childElementName is null or empty.", "childElementName")
			End If

            Dim childElement = (
                From temp In parentElement.Descendants(childElementName)
                Where temp.HasAttributes = True AndAlso temp.Attribute(childAttributeName) IsNot Nothing AndAlso temp.Attribute(childAttributeName).Value = childAttributeValue
                Select temp).FirstOrDefault

			If childElement Is Nothing Then
				Return defaultReturnValue
			Else
				Return childElement.Value
			End If
		End Function

		Public Shared Function StringToXDocument(ByVal fromValue As String) As XDocument
			Return XDocument.Parse(fromValue)
		End Function
	End Class
End Namespace
