Imports Benday.Npr.Common.Repositories
Imports Benday.Npr.Common.DomainModel
Imports Benday.Npr.Common

Namespace Benday.Npr.DataAccess
	Public Class FeedXmlAdapter
		Inherits XmlAdapterBase
		Implements IFeedXmlAdapter
     
        Public Sub New(ByVal factory As IObjectFactory)
            If factory Is Nothing Then
                Throw New ArgumentNullException("factory", "factory is null.")
            End If

            m_ObjectFactoryInstance = factory
        End Sub
        Public Overloads Function StringToXDocument(ByVal fromValue As String) As XDocument Implements IFeedXmlAdapter.StringToXDocument
            Return XmlUtility.StringToXDocument(fromValue)
        End Function
		Private m_ObjectFactoryInstance As IObjectFactory
		Private ReadOnly Property ObjectFactoryInstance As IObjectFactory
			Get
				Return m_ObjectFactoryInstance
			End Get
		End Property

        Public Sub Adapt(ByVal fromValue As String, ByVal toValue As IFeed) Implements IFeedXmlAdapter.Adapt
            If String.IsNullOrEmpty(fromValue) Then
                Throw New ArgumentException("fromValue is null or empty.", "fromValue")
            End If
            If toValue Is Nothing Then
                Throw New ArgumentNullException("toValue", "toValue is null.")
            End If

            Dim doc = StringToXDocument(fromValue)

            toValue.Title = XmlUtility.GetChildElementValue(doc.Root, "title", String.Empty)

            Dim items = doc.Descendants("item")

            Dim toFeedItem As IFeedItem = Nothing

            For Each item In items
                toFeedItem = ObjectFactoryInstance.Create(Of IFeedItem)()

                Adapt(item, toFeedItem)

                toValue.FeedItems.Add(toFeedItem)
            Next item
        End Sub

        Public Sub Adapt(ByVal fromValue As String, ByVal toValue As IFeedItem) Implements IFeedXmlAdapter.Adapt
            If String.IsNullOrEmpty(fromValue) Then
                Throw New ArgumentException("fromValue is null or empty.", "fromValue")
            End If
            If toValue Is Nothing Then
                Throw New ArgumentNullException("toValue", "toValue is null.")
            End If

            Dim doc = StringToXDocument(fromValue)

            Adapt(doc.Root, toValue)
        End Sub

		Private Sub Adapt(ByVal fromValue As XElement, ByVal toValue As IFeedItem)
			If fromValue Is Nothing Then
				Throw New ArgumentNullException("fromValue", "fromValue is null.")
			End If
			If toValue Is Nothing Then
				Throw New ArgumentNullException("toValue", "toValue is null.")
			End If

			toValue.Id = XmlUtility.GetAttributeValue(fromValue, "id", String.Empty)
			toValue.Title = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "title", String.Empty))
			toValue.AdditionalInfo = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "additionalInfo", String.Empty))
		End Sub
	End Class
End Namespace
