Imports Benday.Npr.Common.Repositories
Imports Benday.Npr.Common.DomainModel
Imports Benday.Npr.Common

Namespace Benday.Npr.DataAccess
	Public Class RssFeedXmlAdapter
		Inherits XmlAdapterBase
		Implements IRssFeedXmlAdapter

		Public Sub New(ByVal factory As IObjectFactory)
			If factory Is Nothing Then
				Throw New ArgumentNullException("factory", "factory is null.")
			End If

			m_ObjectFactoryInstance = factory
		End Sub

		Private m_ObjectFactoryInstance As IObjectFactory
		Private ReadOnly Property ObjectFactoryInstance As IObjectFactory
			Get
				Return m_ObjectFactoryInstance
			End Get
		End Property

		Private Sub AssertIsNotNull(ByVal instance As Object, ByVal message As String)
			If instance Is Nothing Then
				Throw New InvalidOperationException(message)
			End If
		End Sub

        Public Sub Adapt(ByVal fromValue As String, ByVal toValue As IStoryList) Implements IRssFeedXmlAdapter.Adapt
            If String.IsNullOrEmpty(fromValue) Then
                Throw New ArgumentException("fromValue is null or empty.", "fromValue")
            End If
            If toValue Is Nothing Then
                Throw New ArgumentNullException("toValue", "toValue is null.")
            End If

            Dim doc = StringToXDocument(fromValue)

            Dim channelElement = XmlUtility.GetChildElement(doc.Root, "channel")

            AssertIsNotNull(channelElement, "Channel element does not exist or could not be found.")

            toValue.Title = XmlUtility.GetChildElementValue(channelElement, "title", String.Empty)
            toValue.Teaser = XmlUtility.GetChildElementValue(channelElement, "description", String.Empty)
            toValue.TeaserMini = toValue.Teaser

            Dim items = channelElement.Descendants("item")

            Dim toStory As IStory = Nothing

            For Each item In items
                toStory = ObjectFactoryInstance.Create(Of IStory)()

                Adapt(item, toStory)

                toValue.Stories.Add(toStory)
            Next item
        End Sub

		Private Sub Adapt(ByVal fromValue As XElement, ByVal toValue As IStory)
			If fromValue Is Nothing Then
				Throw New ArgumentNullException("fromValue", "fromValue is null.")
			End If
			If toValue Is Nothing Then
				Throw New ArgumentNullException("toValue", "toValue is null.")
			End If

			toValue.Title = XmlUtility.GetChildElementValue(fromValue, "title", String.Empty)
			toValue.Teaser = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "description", String.Empty))
			toValue.TeaserMini = toValue.Teaser

			Dim paragraph As IParagraph = ObjectFactoryInstance.Create(Of IParagraph)()
			paragraph.Text = toValue.Teaser

			toValue.Paragraphs.Add(paragraph)

			Dim enclosure = XmlUtility.GetChildElement(fromValue, "enclosure")

			If enclosure Is Nothing Then
				toValue.UrlToMp3 = String.Empty
			Else
				toValue.UrlToMp3 = XmlUtility.GetAttributeValue(enclosure, "url", String.Empty)
			End If

			Dim pubDateAsString = XmlUtility.GetChildElementValue(fromValue, "pubDate", String.Empty)

			If pubDateAsString Is Nothing Then
				toValue.PublishDate = New Date(1970, 1, 1)
				toValue.StoryDate = toValue.PublishDate
			Else
				Dim temp As Date = Date.MinValue

				If Date.TryParse(pubDateAsString, temp) = True Then
					toValue.PublishDate = temp
				Else
					toValue.PublishDate = New Date(1970, 1, 1)
				End If

				toValue.StoryDate = toValue.PublishDate
			End If
		End Sub
	End Class
End Namespace
