Imports Benday.Npr.Common.Repositories
Imports Benday.Npr.Common
Imports Benday.Npr.Common.DomainModel
Imports System.Net
Imports System.IO

Namespace Benday.Npr.DataAccess
	Public Class StoryRepository
		Implements IStoryRepository

		Public Sub New(ByVal factory As IObjectFactory)
			If factory Is Nothing Then
				Throw New ArgumentNullException("factory", "factory is null.")
			End If

			m_ObjectFactoryInstance = factory
		End Sub

		Private m_ObjectFactoryInstance As IObjectFactory
		Private ReadOnly Property ObjectFactoryInstance As IObjectFactory
			Get
				Return m_ObjectFactoryInstance
			End Get
		End Property

        Public Sub GetStories(ByVal action As ReturnResult(Of IStoryList), ByVal id As String) Implements IStoryRepository.GetStories
            If String.IsNullOrEmpty(id) Then
                Throw New ArgumentException("id is null or empty.", "id")
            End If
            If action Is Nothing Then
                Throw New ArgumentNullException("action", "action is null.")
            End If

            If id.StartsWith("RSS_FEED.") = True Then
                ' RSS_FEED.ITS_ALL_POLITICS
                ' RSS_FEED.NPR_HOURLY_NEWS
                GetStoriesForRssFeed(action, id)
            Else
                GetStoriesForProgram(action, id)
            End If
        End Sub

        Private Sub GetStoriesForRssFeed(ByVal action As ReturnResult(Of IStoryList), ByVal id As String)
            Dim url As String

            If id = "RSS_FEED.NPR_HOURLY_NEWS" Then
                url = "http://www.npr.org/rss/podcast.php?id=500005"
            ElseIf id = "RSS_FEED.ITS_ALL_POLITICS" Then
                url = "http://www.npr.org/rss/podcast.php?id=510068"
            ElseIf id = "RSS_FEED.CAR_TALK" Then
                url = "http://www.npr.org/rss/podcast.php?id=510208"
            Else
                Throw New InvalidOperationException(String.Format("Unknown rss feed id '{0}'.", id))
            End If

            Dim request As HttpWebRequest = TryCast(WebRequest.Create(url), HttpWebRequest)

            request.BeginGetResponse(AddressOf ProcessStoriesFromRssFeedResult, New WebRequestReturnResult(Of IStoryList)(request, action))
        End Sub

		Private Sub ProcessStoriesFromRssFeedResult(ByVal asyncResult As IAsyncResult)
			Dim action = TryCast(asyncResult.AsyncState, WebRequestReturnResult(Of IStoryList))

			If action Is Nothing Then
				Throw New InvalidOperationException("User state was the wrong type.")
			End If

			Try
				Dim response = action.Request.EndGetResponse(asyncResult)

				Using stream = New StreamReader(response.GetResponseStream)
					Dim xml As String = stream.ReadToEnd

					Dim adapter = ObjectFactoryInstance.Create(Of IRssFeedXmlAdapter)()

					Dim returnValue = ObjectFactoryInstance.Create(Of IStoryList)()

					adapter.Adapt(xml, returnValue)

                    System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.ReturnResult.Notify(returnValue))
				End Using
			Catch e1 As Exception
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.ReturnResult.Notify(TryCast(Nothing, IStoryList)))
			End Try
		End Sub

		Private Sub GetStoriesForProgram(ByVal action As ReturnResult(Of IStoryList), ByVal id As String)
			Dim url As String = String.Format("http://api.npr.org/query?id={0}&apiKey=MDA2MzU3ODU0MDEyODg2MTQyOTk1OWQ4NQ001", id)

			Dim request As HttpWebRequest = TryCast(WebRequest.Create(url), HttpWebRequest)

			request.BeginGetResponse(AddressOf ProcessStoriesFromQueryResult, New WebRequestReturnResult(Of IStoryList)(request, action))
		End Sub

		Private Sub ProcessStoriesFromQueryResult(ByVal asyncResult As IAsyncResult)
			Dim action = TryCast(asyncResult.AsyncState, WebRequestReturnResult(Of IStoryList))

			If action Is Nothing Then
				Throw New InvalidOperationException("User state was the wrong type.")
			End If

			Try
				Dim response = action.Request.EndGetResponse(asyncResult)

				Using stream = New StreamReader(response.GetResponseStream)
					Dim xml As String = stream.ReadToEnd

					Dim adapter = ObjectFactoryInstance.Create(Of IStoryXmlAdapter)()

					Dim returnValue = ObjectFactoryInstance.Create(Of IStoryList)()

					adapter.Adapt(xml, returnValue)

                    System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.ReturnResult.Notify(returnValue))
				End Using
			Catch e1 As Exception
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.ReturnResult.Notify(TryCast(Nothing, IStoryList)))
			End Try
		End Sub

		Private Sub ProcessMp3ResolveResult(ByVal asyncResult As IAsyncResult)
			Dim action = TryCast(asyncResult.AsyncState, WebRequestReturnResult(Of String))

			If action Is Nothing Then
				Throw New InvalidOperationException("User state was the wrong type.")
			End If

			Try
				Dim response = action.Request.EndGetResponse(asyncResult)

				Using stream = New StreamReader(response.GetResponseStream)
					Dim m3uPlaylistContents As String = stream.ReadToEnd

					Dim playlistReader As New StringReader(m3uPlaylistContents)

					Dim urlToMp3File As String = playlistReader.ReadLine

					Do While urlToMp3File IsNot Nothing AndAlso urlToMp3File.StartsWith("http://") = False
						urlToMp3File = playlistReader.ReadLine
					Loop

                    System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.ReturnResult.Notify(urlToMp3File))
				End Using
			Catch ex As Exception
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.ReturnResult.Notify(ex))
			End Try
		End Sub

        Public Sub ResolveMp3UrlFromM3u(ByVal action As ReturnResult(Of String), ByVal m3uUri As String) Implements IStoryRepository.ResolveMp3UrlFromM3u
            If String.IsNullOrEmpty(m3uUri) Then
                Throw New ArgumentNullException("m3uUri", "m3uUri is null.")
            End If
            If action Is Nothing Then
                Throw New ArgumentNullException("action", "action is null.")
            End If

            Try
                Dim request As HttpWebRequest = TryCast(WebRequest.Create(m3uUri), HttpWebRequest)

                request.BeginGetResponse(AddressOf ProcessMp3ResolveResult, New WebRequestReturnResult(Of String)(request, action))
            Catch e1 As Exception
                Throw
            End Try
        End Sub
	End Class
End Namespace
