Imports Benday.Npr.Common.Repositories
Imports Benday.Npr.Common.DomainModel
Imports Benday.Npr.Common

Namespace Benday.Npr.DataAccess
	Public Class StoryXmlAdapter
		Inherits XmlAdapterBase
		Implements IStoryXmlAdapter

		Public Sub New(ByVal factory As IObjectFactory)
			If factory Is Nothing Then
				Throw New ArgumentNullException("factory", "factory is null.")
			End If

			m_ObjectFactoryInstance = factory
		End Sub

		Private m_ObjectFactoryInstance As IObjectFactory
		Private ReadOnly Property ObjectFactoryInstance As IObjectFactory
			Get
				Return m_ObjectFactoryInstance
			End Get
		End Property

        Public Sub Adapt(ByVal fromValue As String, ByVal toValue As IStoryList) Implements IStoryXmlAdapter.Adapt
            If String.IsNullOrEmpty(fromValue) Then
                Throw New ArgumentException("fromValue is null or empty.", "fromValue")
            End If
            If toValue Is Nothing Then
                Throw New ArgumentNullException("toValue", "toValue is null.")
            End If

            Dim doc = StringToXDocument(fromValue)

            Adapt(doc.Root, toValue)
        End Sub

        Private Sub Adapt(ByVal fromValue As XElement, ByVal toValue As IStoryList)
            If fromValue Is Nothing Then
                Throw New ArgumentNullException("fromValue", "fromValue is null.")
            End If
            If toValue Is Nothing Then
                Throw New ArgumentNullException("toValue", "toValue is null.")
            End If

            toValue.UrlToHtml = XmlUtility.GetChildElementValue(fromValue, "link", "type", "html", String.Empty)
            toValue.UrlToApi = XmlUtility.GetChildElementValue(fromValue, "link", "type", "api", String.Empty)
            toValue.Title = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "title", String.Empty))
            toValue.Teaser = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "teaser", String.Empty))
            toValue.TeaserMini = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "miniTeaser", String.Empty))

            Dim fromStories = fromValue.Descendants("story")

            Dim toStory As IStory

            If fromStories IsNot Nothing Then
                For Each fromStory In fromStories
                    toStory = ObjectFactoryInstance.Create(Of IStory)()

                    Adapt(fromStory, toStory)

                    toValue.Stories.Add(toStory)
                Next fromStory
            End If
        End Sub

        Public Sub Adapt(ByVal fromValue As String, ByVal toValue As IStory) Implements IStoryXmlAdapter.Adapt
            If String.IsNullOrEmpty(fromValue) Then
                Throw New ArgumentException("fromValue is null or empty.", "fromValue")
            End If
            If toValue Is Nothing Then
                Throw New ArgumentNullException("toValue", "toValue is null.")
            End If

            Dim doc = StringToXDocument(fromValue)

            Adapt(doc.Root, toValue)
        End Sub

		Private Sub Adapt(ByVal fromValue As XElement, ByVal toValue As IStory)
			If fromValue Is Nothing Then
				Throw New ArgumentNullException("fromValue", "fromValue is null.")
			End If
			If toValue Is Nothing Then
				Throw New ArgumentNullException("toValue", "toValue is null.")
			End If

			toValue.Id = XmlUtility.GetAttributeValue(fromValue, "id", String.Empty)
			toValue.UrlToHtml = XmlUtility.GetChildElementValue(fromValue, "link", "type", "html", String.Empty)
			toValue.UrlToApi = XmlUtility.GetChildElementValue(fromValue, "link", "type", "api", String.Empty)
			toValue.UrlShort = XmlUtility.GetChildElementValue(fromValue, "link", "type", "short", String.Empty)
			toValue.Title = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "title", String.Empty))
			toValue.Teaser = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "teaser", String.Empty))
			toValue.TeaserMini = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "miniTeaser", String.Empty))
			toValue.Slug = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "slug", String.Empty))
			toValue.StoryDate = XmlUtility.GetChildElementValue(fromValue, "storyDate", Date.MinValue)
			toValue.PublishDate = XmlUtility.GetChildElementValue(fromValue, "pubDate", Date.MinValue)

			AdaptShowInformation(fromValue, toValue)
			AdaptOrganizationInformation(fromValue, toValue)
			AdaptAudioInformation(fromValue, toValue)
			AdaptParagraphs(fromValue, toValue)
			AdaptByline(fromValue, toValue)
		End Sub

		Private Sub AdaptByline(ByVal fromValue As XElement, ByVal toValue As IStory)
			Dim element = XmlUtility.GetChildElement(fromValue, "byline")

			If element Is Nothing Then
				toValue.Author = String.Empty
			Else
				toValue.Author = XmlUtility.GetChildElementValue(element, "name", String.Empty)
			End If
		End Sub

		Private Sub AdaptParagraphs(ByVal fromValue As XElement, ByVal toValue As IStory)
			Dim element = XmlUtility.GetChildElement(fromValue, "text")

			If element Is Nothing Then
				Exit Sub
			Else
				Dim paragraphs = element.Descendants("paragraph")

				Dim toParagraph As IParagraph

				For Each fromParagraph In paragraphs
					toParagraph = ObjectFactoryInstance.Create(Of IParagraph)()

					toParagraph.Text = DataAccessUtility.ReplaceUndisplayableHtml(fromParagraph.Value)

					toValue.Paragraphs.Add(toParagraph)
				Next fromParagraph
			End If
		End Sub

		Private Sub AdaptAudioInformation(ByVal fromValue As XElement, ByVal toValue As IStory)
			Dim audioElement = XmlUtility.GetChildElement(fromValue, "audio")

			If audioElement Is Nothing Then
				toValue.Duration = 0
				toValue.UrlToMp3 = String.Empty
				toValue.UrlToWindowsMedia = String.Empty
			Else
				toValue.Duration = XmlUtility.GetChildElementValue(audioElement, "duration", 0)

				Dim formatElement = XmlUtility.GetChildElement(audioElement, "format")

				If formatElement IsNot Nothing Then
					toValue.UrlToMp3 = XmlUtility.GetChildElementValue(formatElement, "mp3", String.Empty)
					toValue.UrlToWindowsMedia = XmlUtility.GetChildElementValue(formatElement, "wm", String.Empty)
				End If
			End If
		End Sub

		Private Sub AdaptOrganizationInformation(ByVal fromValue As XElement, ByVal toValue As IStory)
			Dim element = XmlUtility.GetChildElement(fromValue, "organization")

			If element Is Nothing Then
				toValue.OrganizationName = String.Empty
				toValue.OrganizationUrl = String.Empty
				toValue.OrganizationAbbreviation = String.Empty
			Else
				toValue.OrganizationName = XmlUtility.GetChildElementValue(element, "name", String.Empty)
				toValue.OrganizationUrl = XmlUtility.GetChildElementValue(element, "website", String.Empty)
				toValue.OrganizationAbbreviation = XmlUtility.GetAttributeValue(element, "orgAbbr", String.Empty)
			End If
		End Sub

		Private Sub AdaptShowInformation(ByVal fromValue As XElement, ByVal toValue As IStory)
			Dim element = XmlUtility.GetChildElement(fromValue, "show")

			If element Is Nothing Then
				toValue.ShowDate = Date.MinValue
				toValue.ShowName = String.Empty
			Else
				toValue.ShowDate = XmlUtility.GetChildElementValue(element, "showDate", Date.Now)
				toValue.ShowName = XmlUtility.GetChildElementValue(element, "program", String.Empty)
			End If
		End Sub
	End Class
End Namespace
