Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Benday.Npr.Common.Repositories
Imports Benday.Npr.Common.DomainModel

Namespace Benday.Npr.Silverlight.UnitTests.DataAccess
	<TestClass>
	Public Class FeedXmlAdapterFixture
		Inherits UnitTestBase

		<TestMethod>
		Public Sub TestDataUtility_LoadTestData_AllPrograms()
			Dim feedXml = TestDataUtility.AllPrograms

			Assert.IsNotNull(feedXml)
			Assert.AreNotEqual(Of String)(String.Empty, feedXml, "Feed xml should not be empty.")
		End Sub

		<TestMethod>
		Public Sub FeedXmlAdapter_ParseAllPrograms()
			Dim adapter As IFeedXmlAdapter = ObjectFactoryInstance.Create(Of IFeedXmlAdapter)()

			Dim toValue As IFeed = ObjectFactoryInstance.Create(Of IFeed)()

			Dim fromValue As String = TestDataUtility.AllPrograms

			adapter.Adapt(fromValue, toValue)

			Assert.AreEqual(Of String)("All Programs", toValue.Title, "Title was wrong.")
			Assert.IsNotNull(toValue.FeedItems, "FeedItems was null.")
			Assert.AreEqual(Of Integer)(13, toValue.FeedItems.Count, "Feed item count was wrong.")

			AssertFeedItem(toValue, 0, "All Songs Considered")
			AssertFeedItem(toValue, 1, "All Things Considered")
			AssertFeedItem(toValue, 2, "Fresh Air from WHYY")
			AssertFeedItem(toValue, 3, "JazzSet with Dee Dee Bridgewater")
			AssertFeedItem(toValue, 12, "World of Opera")
		End Sub

		<TestMethod>
		Public Sub FeedXmlAdapter_ParseAllPrograms_RemovesHtmlFormattingFromAdditionalInfo()
			Dim adapter As IFeedXmlAdapter = ObjectFactoryInstance.Create(Of IFeedXmlAdapter)()

			Dim toValue As IFeed = ObjectFactoryInstance.Create(Of IFeed)()

			Dim fromValue As String = TestDataUtility.AllPrograms

			adapter.Adapt(fromValue, toValue)

			Assert.AreEqual(Of String)("All Programs", toValue.Title, "Title was wrong.")
			Assert.IsNotNull(toValue.FeedItems, "FeedItems was null.")
			Assert.AreEqual(Of Integer)(13, toValue.FeedItems.Count, "Feed item count was wrong.")

            Dim program = (
                From temp In toValue.FeedItems
                Where temp.Title = "Tell Me More"
                Select temp).FirstOrDefault

			Assert.IsNotNull(program, "Could not find the right program for testing.")

			UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(program.AdditionalInfo)
		End Sub

		<TestMethod>
		Public Sub FeedXmlAdapter_StringToXDocument()
			Dim adapter As IFeedXmlAdapter = ObjectFactoryInstance.Create(Of IFeedXmlAdapter)()

			Dim fromValue As String = TestDataUtility.IndividualFeedItem

			Dim toValue = adapter.StringToXDocument(fromValue)

			Assert.IsNotNull(toValue)

			Assert.AreEqual(Of String)("item", toValue.Root.Name.LocalName, "Name was wrong.")
		End Sub

		<TestMethod>
		Public Sub FeedXmlAdapter_ParseFeedItem()
			Dim adapter As IFeedXmlAdapter = ObjectFactoryInstance.Create(Of IFeedXmlAdapter)()

			Dim toValue As IFeedItem = ObjectFactoryInstance.Create(Of IFeedItem)()

			Dim fromValue As String = TestDataUtility.IndividualFeedItem

			adapter.Adapt(fromValue, toValue)

'            
'             * <item id="37" num="1" storycountall="368" storycountmonth="4" storycounttoday="0" type="program">
'    <title>All Songs Considered</title>
'    <altInfo/>
'    <additionalInfo>All Songs Considered is the place to discover music.  The acclaimed weekly program – available as streaming audio, a podcast and a broadcast – melds contemporary music, live concerts, videos and artist interviews to offer a truly eclectic music mix.</additionalInfo>
'  </item>
'             * 

			Assert.AreEqual(Of String)("All Songs Considered", toValue.Title, "Title was wrong.")
			Assert.AreEqual(Of String)("All Songs Considered is the place to discover music.  The acclaimed weekly program – available as streaming audio, a podcast and a broadcast – melds contemporary music, live concerts, videos and artist interviews to offer a truly eclectic music mix.", toValue.AdditionalInfo, "Additional info was wrong.")
			Assert.AreEqual(Of String)("37", toValue.Id, "Id")
		End Sub

		Private Sub AssertFeedItem(ByVal instance As IFeed, ByVal feedItemIndex As Integer, ByVal expectedTitle As String)
			Dim actual As IFeedItem = instance.FeedItems(feedItemIndex)

			Assert.AreEqual(Of String)(expectedTitle, actual.Title, "Title")
		End Sub
	End Class
End Namespace
