Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Benday.Npr.Presentation.Interfaces
Imports Benday.Npr.Common.DomainModel
Imports Benday.Npr.Common
Imports Benday.Npr.Common.Repositories
Imports Benday.Npr.Silverlight.UnitTests.DataAccess
Imports Benday.Npr.Presentation

Namespace Benday.Npr.Silverlight.UnitTests.Presentation
	<TestClass>
	Public Class AllProgramsViewModelFixture
		Inherits UnitTestBase

		Protected Overrides Sub OnTestInitialize()
			m_FeedDomainServiceInstance = Nothing
			m_ViewModelInstance = Nothing
			MyBase.OnTestInitialize()
		End Sub

		Protected Overrides Sub PopulateAdditionalObjectFactoryConfiguration(ByVal configuration As Factories.ObjectFactoryConfiguration)
			configuration.Instances.Add(GetType(IFeedDomainService), FeedDomainServiceInstance)
		End Sub

		Private m_FeedDomainServiceInstance As FeedDomainServiceTester
		Public ReadOnly Property FeedDomainServiceInstance As IFeedDomainService
			Get
				If m_FeedDomainServiceInstance Is Nothing Then
					m_FeedDomainServiceInstance = New FeedDomainServiceTester

					Dim adapter As IFeedXmlAdapter = ObjectFactoryInstance.Create(Of IFeedXmlAdapter)()

					Dim feed As IFeed = ObjectFactoryInstance.Create(Of IFeed)()

					adapter.Adapt(TestDataUtility.AllPrograms, feed)

					m_FeedDomainServiceInstance.ReturnThisFeed = feed
				End If

				Return m_FeedDomainServiceInstance
			End Get
		End Property

		Private m_ViewModelInstance As IAllProgramsViewModel
		Public ReadOnly Property ViewModelInstance As IAllProgramsViewModel
			Get
				If m_ViewModelInstance Is Nothing Then
					m_ViewModelInstance = ObjectFactoryInstance.Create(Of IAllProgramsViewModel)()
				End If

				Return m_ViewModelInstance
			End Get
		End Property

		<TestMethod>
		Public Sub AllProgramsViewModel_Load()
			ViewModelInstance.Load()

			Assert.AreEqual(Of String)("All Programs", ViewModelInstance.Title, "Title was wrong.")

			Assert.IsNotNull(ViewModelInstance.Programs, "Programs was null.")

			Assert.AreEqual(Of Integer)(13, ViewModelInstance.Programs.Count, "Program count was wrong.")

			AssertProgram(ViewModelInstance, 0, "All Songs Considered", 37)
			AssertProgram(ViewModelInstance, 1, "All Things Considered", 2)
			AssertProgram(ViewModelInstance, 2, "Fresh Air from WHYY", 13)
			AssertProgram(ViewModelInstance, 3, "JazzSet with Dee Dee Bridgewater", 20)
			AssertProgram(ViewModelInstance, 12, "World of Opera", 36)
		End Sub

		<TestMethod>
		Public Sub AllProgramsViewModel_SelectedItemChange_UpdatesViewModelLocatorAndTriggersSelectedEvent()
			ViewModelLocator.CreateInstance(ObjectFactoryInstance)

			ViewModelInstance.Load()

			Assert.AreEqual(Of String)("All Programs", ViewModelInstance.Title, "Title was wrong.")

			Assert.IsNotNull(ViewModelInstance.Programs, "Programs was null.")

			Assert.AreEqual(Of Integer)(13, ViewModelInstance.Programs.Count, "Program count was wrong.")

			Assert.IsNull(ViewModelInstance.SelectedItem, "Nothing should be selected yet.")

			Dim expectedSelection As IProgramViewModel = ViewModelInstance.Programs(0)

			Dim tester = New AllProgramsViewModelEventTester(ViewModelInstance)

			Assert.IsNull(ViewModelLocator.Instance.CurrentProgram, "CurrentProgramFeed should be null.")

			ViewModelInstance.SelectedItem = expectedSelection

			Assert.IsTrue(tester.GotSelectedEvent, "Should have gotten selection changed event.")

			Assert.AreSame(expectedSelection, ViewModelLocator.Instance.CurrentProgram, "Wrong instance.")
		End Sub

		Public Class AllProgramsViewModelEventTester
			Public Sub New(ByVal instance As IAllProgramsViewModel)
				If instance Is Nothing Then
					Throw New ArgumentNullException("instance", "instance is null.")
				End If

				m_Instance = instance

				AddHandler m_Instance.OnSelectionChanged, AddressOf m_Instance_OnProgramSelectionChanged
			End Sub

			Private Sub m_Instance_OnProgramSelectionChanged(ByVal sender As Object, ByVal e As EventArgs)
				GotSelectedEvent = True
			End Sub

			Private m_Instance As IAllProgramsViewModel
			Public ReadOnly Property Instance As IAllProgramsViewModel
				Get
					Return m_Instance
				End Get
			End Property

			Public Property GotSelectedEvent As Boolean
		End Class

		Private Sub AssertProgram(ByVal instance As IAllProgramsViewModel, ByVal programIndex As Integer, ByVal expectedTitle As String, ByVal expectedId As Integer)
			Dim actual = instance.Programs(programIndex)

			Assert.AreEqual(Of String)(expectedTitle, actual.Title, "Title")
			Assert.AreEqual(Of String)(expectedId.ToString, actual.Id, "Id")
		End Sub
	End Class
End Namespace
