Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Benday.Npr.Presentation.Interfaces
Imports Benday.Npr.Common.DomainModel
Imports Benday.Npr.Common
Imports Benday.Npr.Common.Repositories
Imports Benday.Npr.Silverlight.UnitTests.DataAccess
Imports Benday.Npr.Presentation

Namespace Benday.Npr.Silverlight.UnitTests.Presentation
	<TestClass>
	Public Class StoryListViewModelFixture
		Inherits UnitTestBase

		Protected Overrides Sub OnTestInitialize()
			m_StoryDomainServiceInstance = Nothing
			m_ViewModelInstance = Nothing
			MyBase.OnTestInitialize()
		End Sub

		Protected Overrides Sub PopulateAdditionalObjectFactoryConfiguration(ByVal configuration As Factories.ObjectFactoryConfiguration)
			configuration.Instances.Add(GetType(IStoryDomainService), StoryDomainServiceInstance)
		End Sub

		Private m_StoryDomainServiceInstance As StoryDomainServiceTester
		Public ReadOnly Property StoryDomainServiceInstance As IStoryDomainService
			Get
				If m_StoryDomainServiceInstance Is Nothing Then
					m_StoryDomainServiceInstance = New StoryDomainServiceTester

					Dim adapter = ObjectFactoryInstance.Create(Of IStoryXmlAdapter)()

					Dim storyList = ObjectFactoryInstance.Create(Of IStoryList)()

					adapter.Adapt(TestDataUtility.AllThingsConsideredStories, storyList)

					m_StoryDomainServiceInstance.ReturnThisStoryList = storyList
				End If

				Return m_StoryDomainServiceInstance
			End Get
		End Property

		Private m_ViewModelInstance As IStoryListViewModel
		Public ReadOnly Property ViewModelInstance As IStoryListViewModel
			Get
				If m_ViewModelInstance Is Nothing Then
					m_ViewModelInstance = ObjectFactoryInstance.Create(Of IStoryListViewModel)()
				End If

				Return m_ViewModelInstance
			End Get
		End Property

		<TestMethod>
		Public Sub StoryListViewModel_Load()
			Dim programId As String = "2"

			ViewModelInstance.Load(programId)

			Assert.AreEqual(Of String)("NPR: All Things Considered", ViewModelInstance.Title, "Title was wrong.")
			Assert.AreEqual(Of String)("http://www.npr.org/programs/all-things-considered/?ft=3&f=2", ViewModelInstance.UrlToHtml, "UrlToHtml")

			Assert.IsNotNull(ViewModelInstance.Stories, "Stories was null.")

			Assert.AreEqual(Of Integer)(10, ViewModelInstance.Stories.Count, "Story count was wrong.")

			AssertProgram(ViewModelInstance, 0, "In Israel, When Is A Jew Not Jewish Enough?", "131216486")
			AssertProgram(ViewModelInstance, 1, "Economic Woes Spur Interest In Refinancing Plan", "131223118")
			AssertProgram(ViewModelInstance, 9, "Deficit Panel Pushes Cuts To Social Security", "131223274")
		End Sub

		Private Sub AssertProgram(ByVal instance As IStoryListViewModel, ByVal storyIndex As Integer, ByVal expectedTitle As String, ByVal expectedId As String)
			Dim actual = instance.Stories(storyIndex)

			Assert.AreEqual(Of String)(expectedTitle, actual.Title, "Title was wrong.")
			Assert.AreEqual(Of String)(expectedId, actual.Id, "Id")
		End Sub

		<TestMethod>
		Public Sub AllProgramsViewModel_SelectedItemChange_UpdatesViewModelLocatorAndTriggersSelectedEvent()
			ViewModelLocator.CreateInstance(ObjectFactoryInstance)

			ViewModelInstance.Load("2")

			Assert.AreNotEqual(Of Integer)(0, ViewModelInstance.Stories.Count, "Story count was wrong.")

			Assert.IsNull(ViewModelInstance.SelectedItem, "Nothing should be selected yet.")

			Dim expectedSelection = ViewModelInstance.Stories(0)

			Dim tester = New StoryListViewModelEventTester(ViewModelInstance)

			Assert.IsNull(ViewModelLocator.Instance.CurrentStory, "CurrentStory should be null.")

			ViewModelInstance.SelectedItem = expectedSelection

			Assert.IsTrue(tester.GotSelectedEvent, "Should have gotten selection changed event.")

			Assert.AreSame(expectedSelection, ViewModelLocator.Instance.CurrentStory, "Wrong instance.")
		End Sub

		Public Class StoryListViewModelEventTester
			Public Sub New(ByVal instance As IStoryListViewModel)
				If instance Is Nothing Then
					Throw New ArgumentNullException("instance", "instance is null.")
				End If

				m_Instance = instance

				AddHandler m_Instance.OnSelectionChanged, AddressOf m_Instance_OnSelectionChanged
			End Sub

			Private Sub m_Instance_OnSelectionChanged(ByVal sender As Object, ByVal e As EventArgs)
				GotSelectedEvent = True
			End Sub

			Private m_Instance As IStoryListViewModel
			Public ReadOnly Property Instance As IStoryListViewModel
				Get
					Return m_Instance
				End Get
			End Property

			Public Property GotSelectedEvent As Boolean
		End Class
	End Class
End Namespace
