Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Benday.Npr.Presentation.Interfaces
Imports Benday.Npr.Common.DomainModel
Imports Benday.Npr.Common
Imports Benday.Npr.Common.Repositories
Imports Benday.Npr.Silverlight.UnitTests.DataAccess
Imports System.Collections.ObjectModel
Imports Benday.Npr.Presentation

Namespace Benday.Npr.Silverlight.UnitTests.Presentation
	<TestClass>
	Public Class StoryViewModelFixture
		Inherits UnitTestBase

		Public Sub New()
		End Sub

		Protected Overrides Sub OnTestInitialize()
			m_ViewModelInstance = Nothing
			MyBase.OnTestInitialize()
		End Sub

		Private m_StoryModelInstance As IStory
		Private ReadOnly Property StoryModelInstance As IStory
			Get
				If m_StoryModelInstance Is Nothing Then
					Dim adapter = ObjectFactoryInstance.Create(Of IStoryXmlAdapter)()

					Dim story = ObjectFactoryInstance.Create(Of IStory)()

					adapter.Adapt(TestDataUtility.IndividualStory, story)

					m_StoryModelInstance = story
				End If

				Return m_StoryModelInstance
			End Get
		End Property

		Private m_ViewModelInstance As IStoryViewModel
		Public ReadOnly Property ViewModelInstance As IStoryViewModel
			Get
				If m_ViewModelInstance Is Nothing Then
					m_ViewModelInstance = ObjectFactoryInstance.Create(Of IStoryViewModel)()
				End If

				Return m_ViewModelInstance
			End Get
		End Property

		<TestMethod>
		Public Sub StoryViewModel_Load()
			Dim programId As String = "2"

			ViewModelInstance.Load(StoryModelInstance)

			AssertAreEqual(StoryModelInstance, ViewModelInstance)
		End Sub

		Private Sub AssertAreEqual(ByVal expected As IStory, ByVal actual As IStoryViewModel)
			If expected Is Nothing Then
				Throw New ArgumentNullException("expected", "expected is null.")
			End If
			If actual Is Nothing Then
				Throw New ArgumentNullException("actual", "actual is null.")
			End If

			Assert.AreEqual(Of String)(expected.Id, actual.Id, "Id")
			Assert.AreEqual(Of String)(expected.UrlToHtml, actual.UrlToHtml, "UrlToHtml")
			Assert.AreEqual(Of String)(expected.UrlToApi, actual.UrlToApi, "UrlToApi")
			Assert.AreEqual(Of String)(expected.UrlShort, actual.UrlShort, "UrlShort")
			Assert.AreEqual(Of String)(expected.Title, actual.Title, "Title")
			Assert.AreEqual(Of String)(expected.Teaser, actual.Teaser, "Teaser")
			Assert.AreEqual(Of String)(expected.TeaserMini, actual.TeaserMini, "TeaserMini")
			Assert.AreEqual(Of String)(expected.Slug, actual.Slug, "Slug")
			Assert.AreEqual(Of Date)(expected.StoryDate, actual.StoryDate, "StoryDate")
			Assert.AreEqual(Of Date)(expected.PublishDate, actual.PublishDate, "PublishDate")
			Assert.AreEqual(Of String)(expected.ShowName, actual.ShowName, "ShowName")
			Assert.AreEqual(Of String)(expected.OrganizationName, actual.OrganizationName, "OrganizationName")
			Assert.AreEqual(Of String)(expected.OrganizationAbbreviation, actual.OrganizationAbbreviation, "OrganizationAbbreviation")
			Assert.AreEqual(Of String)(expected.OrganizationUrl, actual.OrganizationUrl, "OrganizationUrl")
			Assert.AreEqual(Of Integer)(expected.Duration, actual.Duration, "Duration")
			Assert.AreEqual(Of String)(expected.UrlToMp3, actual.UrlToMp3, "UrlToMp3")
			Assert.AreEqual(Of String)(expected.UrlToWindowsMedia, actual.UrlToWindowsMedia, "UrlToWindowsMedia")
			Assert.AreEqual(Of String)(expected.Author, actual.Author, "Author")

			AssertParagraphs(expected.Paragraphs, actual.Paragraphs)
		End Sub

		Private Sub AssertParagraphs(ByVal expected As IList(Of IParagraph), ByVal actual As ObservableCollection(Of IParagraphViewModel))
			If expected Is Nothing Then
				Throw New ArgumentNullException("expected", "expected is null.")
			End If
			If actual Is Nothing Then
				Throw New ArgumentNullException("actual", "actual is null.")
			End If

			Assert.AreEqual(Of Integer)(expected.Count, actual.Count, "Count")

			For i  = 0 To expected.Count - 1
				AssertAreEqual(expected(i), actual(i))
			Next i
		End Sub

		Private Sub AssertAreEqual(ByVal expected As IParagraph, ByVal actual As IParagraphViewModel)
			If expected Is Nothing Then
				Throw New ArgumentNullException("expected", "expected is null.")
			End If
			If actual Is Nothing Then
				Throw New ArgumentNullException("actual", "actual is null.")
			End If

			Assert.AreEqual(Of String)(expected.Text, actual.Text, "Text")
		End Sub
	End Class
End Namespace
