Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Navigation
Imports System.Windows.Shapes
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports Benday.Npr.Presentation
Imports System.IO.IsolatedStorage
Imports System.IO
Imports Microsoft.Advertising.Mobile.UI
Imports System.Net.NetworkInformation
Imports Microsoft.Phone.Net.NetworkInformation
Imports Benday.Npr.Common
Imports Benday.Npr.Presentation.Interfaces

Namespace Benday.Npr.Phone
	Partial Public Class App
		Inherits Application

		Private Const STATE_DATA_OBJECT_KEY As String = "dataObject"
		Private Const STATE_DATA_FILENAME As String = "dataObject.txt"

		''' <summary>
		''' Provides easy access to the root frame of the Phone Application.
		''' </summary>
		''' <returns>The root frame of the Phone Application.</returns>
        Private _rootFrame As PhoneApplicationFrame
		Public Property RootFrame As PhoneApplicationFrame
			Get
                Return _rootFrame
			End Get
			Private Set(ByVal value As PhoneApplicationFrame)
                _rootFrame = value
			End Set
		End Property

		''' <summary>
		''' Constructor for the Application object.
		''' </summary>
		Public Sub New()
			' Global handler for uncaught exceptions. 
			AddHandler UnhandledException, AddressOf Application_UnhandledException

			' Show graphics profiling information while debugging.
			If Debugger.IsAttached Then
				' Display the current frame rate counters.
				Application.Current.Host.Settings.EnableFrameRateCounter = True

				' Show the areas of the app that are being redrawn in each frame.
                'Application.Current.Host.Settings.EnableRedrawRegions = True

				' Enable non-production analysis visualization mode, 
				' which shows areas of a page that are being GPU accelerated with a colored overlay.
                'Application.Current.Host.Settings.EnableCacheVisualization = True
			End If

			Benday.Npr.Presentation.ViewModelLocator.CreateInstance(New Benday.Npr.Factories.ObjectFactory)

			Dim instance As ViewModelLocator = Benday.Npr.Presentation.ViewModelLocator.Instance

			Me.Resources.Add("ViewModelLocatorInstance", instance)

			' Standard Silverlight initialization
			InitializeComponent()

			' Phone-specific initialization
			InitializePhoneApplication()

			' disable screen timeout
			PhoneApplicationService.Current.ApplicationIdleDetectionMode = IdleDetectionMode.Disabled

			AddHandler NetworkChange.NetworkAddressChanged, AddressOf NetworkChange_NetworkAddressChanged

			RefreshIsNetworkAvailable()
		End Sub

		Private Sub NetworkChange_NetworkAddressChanged(ByVal sender As Object, ByVal e As EventArgs)
			RefreshIsNetworkAvailable()
		End Sub

		Public Sub RefreshIsNetworkAvailable()
			Dim networkInterface As NetworkInterfaceType = Microsoft.Phone.Net.NetworkInformation.NetworkInterface.NetworkInterfaceType

			Dim isNetworkAvailable As Boolean = False

			If networkInterface = NetworkInterfaceType.MobileBroadbandCdma OrElse networkInterface = NetworkInterfaceType.MobileBroadbandGsm OrElse networkInterface = NetworkInterfaceType.Wireless80211 OrElse networkInterface = NetworkInterfaceType.Ethernet Then
				isNetworkAvailable = True
			End If

			ViewModelLocator.Instance.NotifyNetworkAvailabilityChange(isNetworkAvailable)

			If isNetworkAvailable = False Then
				ApplicationUtility.NavigateToMainPage()
			End If
		End Sub

		' Code to execute when the application is launching (eg, from Start)
		' This code will not execute when the application is reactivated
		Private Sub Application_Launching(ByVal sender As Object, ByVal e As LaunchingEventArgs)
		End Sub

		' Code to execute when the application is activated (brought to foreground)
		' This code will not execute when the application is first launched
		Private Sub Application_Activated(ByVal sender As Object, ByVal e As ActivatedEventArgs)

            '         ' Check to see if the key for the application state data is in the State dictionary.

            '         ' If (PhoneApplicationService.Current.State.ContainsKey(STATE_DATA_OBJECT_KEY)) Then
            'Private Sub New(PhoneApplicationService.Current.State.ContainsKey(STATE_DATA_OBJECT_KEY) ByVal As )
            '         ' If it exists, assign the data to the application member variable.
            '         ViewModelLocator.Instance = TryCast(PhoneApplicationService.Current.StateSTATE_DATA_OBJECT_KEY, ViewModelLocator)
            '     End Sub


            RefreshIsNetworkAvailable()

            Try
                Dim stateCollection As IDictionary(Of String, Object) = PhoneApplicationService.Current.State

                Dim resumeAction As String = TryCast(stateCollection(ViewModelSerializerBase.SERIALIZATION_KEY_RESUME_ACTION), String)

                If resumeAction = ViewModelSerializerBase.SERIALIZATION_KEY_RESUME_ACTION_STORY Then
                    CType(New StoryListViewModelSerializer, StoryListViewModelSerializer).Deserialize()
                    CType(New StoryViewModelSerializer, StoryViewModelSerializer).Deserialize()
                ElseIf resumeAction = ViewModelSerializerBase.SERIALIZATION_KEY_RESUME_ACTION_PROGRAM Then
                    CType(New StoryListViewModelSerializer, StoryListViewModelSerializer).Deserialize()
                    CType(New ProgramViewModelSerializer, ProgramViewModelSerializer).Deserialize()
                ElseIf resumeAction = ViewModelSerializerBase.SERIALIZATION_KEY_RESUME_ACTION_HOURLY_NEWS Then
                    CType(New StoryListViewModelSerializer, StoryListViewModelSerializer).Deserialize()
                Else
                    ApplicationUtility.NavigateToMainPage()
                End If
            Catch ex As Exception
                Debug.WriteLine(ex.ToString)
                ApplicationUtility.NavigateToMainPage()
            End Try
        End Sub

        ' Code to execute when the application is deactivated (sent to background)
        ' This code will not execute when the application is closing
        Private Sub Application_Deactivated(ByVal sender As Object, ByVal e As DeactivatedEventArgs)

            '' Ensure that required application state is persisted here.
            '           ' If there is data in the application member variable...
            '           If ViewModelLocator.Instance IsNot Nothing Then
            '               ' Store it in the State dictionary.
            '               PhoneApplicationService.Current.StateSTATE_DATA_OBJECT_KEY = ViewModelLocator.Instance
            '           End If


            Try
                ViewModelSerializerBase.ClearState()

                If IsDisplayingStory() = True Then
                    CType(New StoryListViewModelSerializer, StoryListViewModelSerializer).Serialize()
                    CType(New StoryViewModelSerializer, StoryViewModelSerializer).Serialize()

                    ViewModelSerializerBase.SetResumeActionToStory()
                ElseIf IsDisplayingProgram() = True Then
                    CType(New StoryListViewModelSerializer, StoryListViewModelSerializer).Serialize()
                    CType(New ProgramViewModelSerializer, ProgramViewModelSerializer).Serialize()

                    ViewModelSerializerBase.SetResumeActionToProgram()
                ElseIf IsDisplayingHourlyNews() = True Then
                    CType(New StoryListViewModelSerializer, StoryListViewModelSerializer).Serialize()

                    ViewModelSerializerBase.SetResumeActionToHourlyNews()
                End If
            Catch ex As Exception
                Debug.WriteLine(ex.ToString)
            End Try
        End Sub

        Private Function IsDisplayingProgram() As Boolean
            Return ApplicationUtility.DoesCurrentRootVisualSourceContain("StoryListPage.xaml")
        End Function

        Private Function IsDisplayingHourlyNews() As Boolean
            Return ApplicationUtility.DoesCurrentRootVisualSourceContain("HourlyNewsPage.xaml")
        End Function

        Private Function IsDisplayingStory() As Boolean
            Return ApplicationUtility.DoesCurrentRootVisualSourceContain("StoryDetailsPage.xaml")
        End Function

        ' Code to execute when the application is closing (eg, user hit Back)
        ' This code will not execute when the application is deactivated
        Private Sub Application_Closing(ByVal sender As Object, ByVal e As ClosingEventArgs)
        End Sub

        ' Code to execute if a navigation fails
        Private Sub RootFrame_NavigationFailed(ByVal sender As Object, ByVal e As NavigationFailedEventArgs)
            If Debugger.IsAttached Then
                ' A navigation has failed; break into the debugger
                Debugger.Break()
            End If
        End Sub

        ' Code to execute on Unhandled Exceptions
        Private Sub Application_UnhandledException(ByVal sender As Object, ByVal e As ApplicationUnhandledExceptionEventArgs)
            If Debugger.IsAttached Then
                ' An unhandled exception has occurred; break into the debugger
                Debugger.Break()
            Else
                MessageBox.Show(e.ExceptionObject.Message)
            End If
        End Sub

#Region "Phone application initialization"
        ' Avoid double-initialization
        Private phoneApplicationInitialized As Boolean = False

        ' Do not add any additional code to this method
        Private Sub InitializePhoneApplication()
            If phoneApplicationInitialized Then
                Exit Sub
            End If

            ' Create the frame but don't set it as RootVisual yet; this allows the splash
            ' screen to remain active until the application is ready to render.
            RootFrame = New TransitionFrame
            AddHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication

            ' Handle navigation failures
            AddHandler RootFrame.NavigationFailed, AddressOf RootFrame_NavigationFailed

            ' Ensure we don't initialize again
            phoneApplicationInitialized = True
        End Sub

        ' Do not add any additional code to this method
        Private Sub CompleteInitializePhoneApplication(ByVal sender As Object, ByVal e As NavigationEventArgs)
            ' Set the root visual to allow the application to render
            If RootVisual IsNot RootFrame Then
                RootVisual = RootFrame
            End If

            ' Remove this handler since it is no longer needed
            RemoveHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication
        End Sub
#End Region

    End Class
End Namespace
