Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports Microsoft.Phone.Controls
Imports Benday.Npr.Presentation
Imports Microsoft.Phone.Tasks
Imports Microsoft.Phone.Shell
Imports Benday.Npr.Presentation.Interfaces
Imports Microsoft.Xna.Framework.Media

Namespace Benday.Npr.Phone
	Public Class ApplicationBarBindingHelper
		Public Sub New(ByVal bar As IApplicationBar, ByVal viewModel As IMediaPlaybackViewModel)
			If bar Is Nothing Then
				Throw New ArgumentNullException("bar", "bar is null.")
			End If
			If viewModel Is Nothing Then
				Throw New ArgumentNullException("viewModel", "viewModel is null.")
			End If

			m_BarInstance = bar
			m_ViewModelInstance = viewModel

			AddHandler m_ViewModelInstance.PropertyChanged, AddressOf m_ViewModelInstance_PropertyChanged
			AddHandler PlayPauseButton.Click, AddressOf PlayPauseButton_Click

			AddHandler MediaPlayer.MediaStateChanged, AddressOf MediaPlayer_MediaStateChanged

			SyncToMediaPlayer()
		End Sub

		Private Sub SyncToMediaPlayer()
			If MediaPlayer.GameHasControl = True Then
				Dim state = MediaPlayer.State

				If state = MediaState.Stopped Then
					ViewModelInstance.SetState("Stopped")
				ElseIf state = MediaState.Playing Then
					ViewModelInstance.SetState("Playing")
				ElseIf state = MediaState.Paused Then
					ViewModelInstance.SetState("Paused")
				End If
			Else
				ViewModelInstance.IsPlayable = False
				BarInstance.IsVisible = False
			End If
		End Sub

		Private Sub MediaPlayer_MediaStateChanged(ByVal sender As Object, ByVal e As EventArgs)
			SyncToMediaPlayer()
		End Sub

		Private Sub PlayPauseButton_Click(ByVal sender As Object, ByVal e As EventArgs)
			If MediaPlayer.GameHasControl = True Then
				Dim state = MediaPlayer.State

				If state = MediaState.Stopped Then
					' do nothing
				ElseIf state = MediaState.Playing Then
					MediaPlayer.Pause()
				ElseIf state = MediaState.Paused Then
					MediaPlayer.Resume()
				End If
			Else
				' do nothing
			End If
		End Sub

		Private Sub m_ViewModelInstance_PropertyChanged(ByVal sender As Object, ByVal e As System.ComponentModel.PropertyChangedEventArgs)
			Dim propertyName = e.PropertyName

			If propertyName Is Nothing OrElse propertyName Is String.Empty Then
				Exit Sub
			ElseIf propertyName Is "IsPlayable" Then
				IsPlayableChanged()
			ElseIf propertyName Is "PlayPauseIconUri" Then
				PlayPauseIconUriChanged()
			ElseIf propertyName Is "PlayPauseText" Then
				PlayPauseTextChanged()
			End If
		End Sub

		Private m_BarInstance As IApplicationBar
		Public ReadOnly Property BarInstance As IApplicationBar
			Get
				Return m_BarInstance
			End Get
		End Property

		Private m_ViewModelInstance As IMediaPlaybackViewModel
		Public ReadOnly Property ViewModelInstance As IMediaPlaybackViewModel
			Get
				Return m_ViewModelInstance
			End Get
		End Property

		Private Sub IsPlayableChanged()
			BarInstance.IsVisible = ViewModelInstance.IsPlayable
		End Sub

		Private ReadOnly Property PlayPauseButton As ApplicationBarIconButton
			Get
				If BarInstance.Buttons.Count >= 1 Then
					Return TryCast(BarInstance.Buttons(0), ApplicationBarIconButton)
				Else
					Return Nothing
				End If
			End Get
		End Property

		Private Sub PlayPauseIconUriChanged()
			Dim button = PlayPauseButton

			If button IsNot Nothing Then
				button.IconUri = New Uri(ViewModelInstance.PlayPauseIconUri, UriKind.Relative)
			End If
		End Sub

		Private Sub PlayPauseTextChanged()
			Dim button = PlayPauseButton

			If button IsNot Nothing Then
				button.Text = ViewModelInstance.PlayPauseText
			End If
		End Sub
	End Class
End Namespace
