Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports Microsoft.Phone.Controls
Imports Benday.Npr.Presentation
Imports Microsoft.Phone.Tasks
Imports Benday.Npr.Presentation.Interfaces

Namespace Benday.Npr.Phone
	Partial Public Class MainPage
		Inherits PhoneApplicationPage

		Private m_ViewModel As Benday.Npr.Presentation.Interfaces.IAllProgramsViewModel
		' Constructor
		Public Sub New()
			InitializeComponent()

			AddHandler Loaded, AddressOf MainPage_Loaded

			PopulateVersionNumber()
		End Sub

		Private Sub PopulateVersionNumber()
			m_textVersion.Text = "v1.4.2"
		End Sub

		Protected Overrides Sub OnNavigatedTo(ByVal e As System.Windows.Navigation.NavigationEventArgs)
            ' CheckApplicationBarHelper()
			MyBase.OnNavigatedTo(e)
		End Sub

		Private Sub CheckApplicationBarHelper()
			If m_ApplicationBarHelper Is Nothing Then
				m_ApplicationBarHelper = New ApplicationBarBindingHelper(Me.ApplicationBar, ViewModelLocator.Instance.AllPrograms.MediaPlayback)
			End If
		End Sub

		' Handle selection changed on ListBox
		Private Sub MainListBox_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
			' If selected index is -1 (no selection) do nothing
			If MainListBox.SelectedIndex = -1 Then
				Exit Sub
			End If

			' Navigate to the new page
			NavigationService.Navigate(New Uri("/StoryListPage.xaml?selectedItem=" & MainListBox.SelectedIndex, UriKind.Relative))

			' Reset selected index to -1 (no selection)
			MainListBox.SelectedIndex = -1
		End Sub

		Private m_ApplicationBarHelper As ApplicationBarBindingHelper
		Public ReadOnly Property ApplicationBarHelper As ApplicationBarBindingHelper
			Get
				Return m_ApplicationBarHelper
			End Get
		End Property

		' Load data for the ViewModel Items
		Private Sub MainPage_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			m_ViewModel = ViewModelLocator.Instance.AllPrograms

            ' CheckApplicationBarHelper()

			Me.DataContext = m_ViewModel

			If m_ViewModel.Programs.Count = 0 Then
				m_ViewModel.Load()
			End If

			AddHandler m_ViewModel.OnSelectionChanged, AddressOf viewModel_OnSelectionChanged
		End Sub

		Private Sub viewModel_OnSelectionChanged(ByVal sender As Object, ByVal e As EventArgs)
			NavigateToProgram()
		End Sub

		Private Sub NavigateToProgram()
			If m_ViewModel IsNot Nothing AndAlso m_ViewModel.SelectedItem IsNot Nothing Then
				If m_ViewModel.SelectedItem.Id = "RSS_FEED.NPR_HOURLY_NEWS" Then
					NavigationService.Navigate(New Uri("/HourlyNewsPage.xaml", UriKind.Relative))
				Else
					NavigationService.Navigate(New Uri("/StoryListPage.xaml", UriKind.Relative))
				End If
			End If
		End Sub

		Private Sub StackPanel_MouseLeftButtonDown(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
		End Sub

		Private Sub StackPanel_MouseLeftButtonUp(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
			Dim senderAsStackPanel As StackPanel = TryCast(sender, StackPanel)

			If senderAsStackPanel IsNot Nothing Then
				Dim programForThisItem As IProgramViewModel = TryCast(senderAsStackPanel.DataContext, IProgramViewModel)

				If programForThisItem IsNot Nothing AndAlso m_ViewModel IsNot Nothing AndAlso m_ViewModel.SelectedItem IsNot Nothing AndAlso m_ViewModel.SelectedItem Is programForThisItem Then
					' the current program is being clicked again
					NavigateToProgram()
				End If
			End If
		End Sub

		Private Sub HyperlinkButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			Dim senderAsHyperLink As HyperlinkButton = TryCast(sender, HyperlinkButton)

			If senderAsHyperLink IsNot Nothing Then
				Dim task = New WebBrowserTask
				task.URL = TryCast(senderAsHyperLink.Tag, String)

				task.Show()
			End If
		End Sub
	End Class
End Namespace
