Imports System.Windows
Imports System.Windows.Controls
Imports Microsoft.Xna.Framework.Media
Imports System.Threading
Imports Benday.Npr.Presentation
Imports Benday.Npr.Presentation.Interfaces
Imports Microsoft.Advertising.Mobile.UI
Imports System.Windows.Threading
Imports Microsoft.Phone.BackgroundAudio

Namespace Benday.Npr.Phone
	Partial Public Class MediaPlaybackControl
		Inherits UserControl

		Private m_timer As DispatcherTimer

		Public Sub New()
			InitializeComponent()

			If GalaSoft.MvvmLight.ViewModelBase.IsInDesignModeStatic = False Then
				m_timer = New DispatcherTimer
				m_timer.Interval = TimeSpan.FromSeconds(0.5)
				AddHandler m_timer.Tick, AddressOf UpdateState

				AddHandler BackgroundAudioPlayer.Instance.PlayStateChanged, AddressOf Instance_PlayStateChanged

				m_ButtonImage.DataContext = ViewModelInstance

				Dim isInTestMode As Boolean = False

				If isInTestMode = False Then
					m_adControl.AdUnitId = "66946"
					m_adControl.ApplicationId = "d38cc7ed-c6b4-4d0c-8595-45bb22eb2801"
				Else
					m_adControl.AdUnitId="Image480_80"
					m_adControl.ApplicationId = "test_client"
				End If
			End If
		End Sub

		Private Sub UpdateState(ByVal sender As Object, ByVal e As EventArgs)
            'txtState.Text = String.Format("State: {0}", BackgroundAudioPlayer.Instance.PlayerState)

			If BackgroundAudioPlayer.Instance.Track IsNot Nothing Then
                'txtTrack.Text = String.Format("Track: {0}", BackgroundAudioPlayer.Instance.Track.Title)

				' Set the current position on the ProgressBar.
				m_ProgressBar.Value = BackgroundAudioPlayer.Instance.Position.TotalSeconds

				' Update the current playback position.
				Dim position As New TimeSpan
				position = BackgroundAudioPlayer.Instance.Position
				m_LabelCurrentPosition.Text = String.Format("{0:d2}:{1:d2}:{2:d2}", position.Hours, position.Minutes, position.Seconds)
			End If
		End Sub

		Private Sub Instance_PlayStateChanged(ByVal sender As Object, ByVal e As EventArgs)
			Select Case BackgroundAudioPlayer.Instance.PlayerState
				Case PlayState.Playing
					' Update the UI.
					m_ProgressBar.IsIndeterminate = False
					m_ProgressBar.Maximum = BackgroundAudioPlayer.Instance.Track.Duration.TotalSeconds
                    'UpdateButtons(true, false, true, true)
					UpdateState(Nothing, Nothing)

					' Start the timer for updating the UI.
					m_timer.Start()

				Case PlayState.Paused
					' Update the UI.
                    'UpdateButtons(true, true, false, true)
					UpdateState(Nothing, Nothing)

					' Stop the timer for updating the UI.
					m_timer.Stop()
			End Select
		End Sub

		Private ReadOnly Property ViewModelInstance As IMediaPlaybackViewModel
			Get
				Return ViewModelLocator.Instance.AllPrograms.MediaPlayback
			End Get
		End Property

		Private Sub SyncToMediaPlayer()
			If MediaPlayer.GameHasControl = True Then
				Dim state As PlayState = BackgroundAudioPlayer.Instance.PlayerState

                If state = PlayState.Stopped Then
                    ViewModelInstance.SetState("Stopped")

                    m_PlaybackContentControl.IsEnabled = False
                ElseIf state = PlayState.Playing Then
                    ViewModelInstance.SetState("Playing")

                    m_PlaybackContentControl.IsEnabled = True
                ElseIf state = PlayState.Paused Then
                    ViewModelInstance.SetState("Paused")

                    m_PlaybackContentControl.IsEnabled = True
                End If
			Else
				ViewModelInstance.IsPlayable = False

				m_PlaybackContentControl.IsEnabled = False
			End If
		End Sub

		Private Sub SyncUi(ByVal sender As Object, ByVal e As EventArgs)
			SyncUi()
		End Sub

		Private Sub SyncUi()
			If MediaPlayer.GameHasControl = True Then
				m_LabelCurrentPosition.Text = New Date(MediaPlayer.PlayPosition.Ticks).ToString("m:ss")
			End If

			SyncToMediaPlayer()
		End Sub

		Private Sub Play_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			HandlePlayPauseClick()
		End Sub

		Private Sub Pause_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			HandlePlayPauseClick()
		End Sub

		Private Sub HandlePlayPauseClick()
			If MediaPlayer.GameHasControl = True Then
				Dim state As PlayState = BackgroundAudioPlayer.Instance.PlayerState

                If state = PlayState.Stopped Then
                    ' do nothing
                ElseIf state = PlayState.Playing Then
                    BackgroundAudioPlayer.Instance.Pause()
                ElseIf state = PlayState.Paused Then
                    BackgroundAudioPlayer.Instance.Play()
                End If
			Else
				' do nothing
			End If
		End Sub
	End Class
End Namespace
