Imports Benday.Npr.Presentation
Imports Microsoft.Phone.Shell
Imports Benday.Npr.Presentation.Interfaces
Imports Benday.Npr.Common

Namespace Benday.Npr.Phone
	Public Class ProgramViewModelSerializer
		Inherits ViewModelSerializerBase

		Public Sub New()
		End Sub

		Public Sub Deserialize()
			If HasStateValue(ViewModelSerializerBase.SERIALIZATION_KEY_PROGRAM) = False Then
				Throw New InvalidOperationException("Could not find serialized program xml in state.")
			Else
				Try
					Dim builder As New System.Text.StringBuilder
					Dim serializedProgramXml As String = GetStateValue(ViewModelSerializerBase.SERIALIZATION_KEY_PROGRAM)

					If String.IsNullOrEmpty(serializedProgramXml) = True Then
						Throw New InvalidOperationException("Serialized program xml was null or empty.")
					End If

					Dim document = XmlUtility.StringToXDocument(serializedProgramXml)

					Dim program As IProgramViewModel = ViewModelLocator.Instance.ObjectFactoryInstance.Create(Of IProgramViewModel)()

					PopulateFromDocument(document, program)

					ViewModelLocator.Instance.CurrentProgram = ViewModelLocator.Instance.CurrentProgram
				Catch e1 As Exception
					Throw
				End Try
			End If
		End Sub

		Private Sub PopulateFromDocument(ByVal document As XDocument, ByVal toValue As IProgramViewModel)
			Dim root = document.Root

			toValue.Id = XmlUtility.GetChildElementValue(root, "Id", String.Empty)
			toValue.AdditionalInformation = XmlUtility.GetChildElementValue(root, "AdditionalInformation", String.Empty)
			toValue.Title = XmlUtility.GetChildElementValue(root, "Title", String.Empty)
		End Sub

		Public Sub Serialize()
			If ViewModelLocator.Instance.CurrentProgram IsNot Nothing Then
				Serialize(ViewModelLocator.Instance.CurrentProgram)
			End If
		End Sub

		Private Sub Serialize(ByVal fromValue As IProgramViewModel)
			Dim document = XmlUtility.StringToXDocument("<program />")

			WriteToDocument(document, fromValue)

			SetStateValue(SERIALIZATION_KEY_PROGRAM, document.ToString)
		End Sub

		Private Sub WriteToDocument(ByVal document As XDocument, ByVal story As IProgramViewModel)
			Dim root = document.Root

			XmlUtility.SetChildElement(root, "Id", story.Id)
			XmlUtility.SetChildElement(root, "Title", story.Title)
			XmlUtility.SetChildElement(root, "AdditionalInformation", story.AdditionalInformation)
		End Sub
	End Class
End Namespace
