Imports Benday.Npr.Presentation
Imports Microsoft.Phone.Shell
Imports Benday.Npr.Presentation.Interfaces
Imports Benday.Npr.Common

Namespace Benday.Npr.Phone
	Public Class StoryListViewModelSerializer
		Inherits ViewModelSerializerBase

		Public Sub Deserialize()
			If HasStateValue(ViewModelSerializerBase.SERIALIZATION_KEY_STORY_LIST) = False Then
				Throw New InvalidOperationException("Could not find story list xml in state.")
			Else
				Try
					Dim builder As New System.Text.StringBuilder
					Dim serializedStoryListXml As String = GetStateValue(ViewModelSerializerBase.SERIALIZATION_KEY_STORY_LIST)

					If String.IsNullOrEmpty(serializedStoryListXml) = True Then
						Throw New InvalidOperationException("Story list xml is null or empty.")
					End If

					Dim document = XmlUtility.StringToXDocument(serializedStoryListXml)

					Dim storyList As IStoryListViewModel = ViewModelLocator.Instance.StoryListInstance

					PopulateFromDocument(document, storyList)
				Catch e1 As Exception
					Throw
				End Try
			End If
		End Sub

		Private Sub PopulateFromDocument(ByVal document As XDocument, ByVal toValue As IStoryListViewModel)
			Dim root = document.Root

			toValue.Id = XmlUtility.GetChildElementValue(root, "Id", String.Empty)
			toValue.UrlToHtml = XmlUtility.GetChildElementValue(root, "UrlToHtml", String.Empty)
			toValue.Title = XmlUtility.GetChildElementValue(root, "Title", String.Empty)

			Dim stories As IEnumerable(Of XElement) = root.Elements("story")

			Dim serializer = New StoryViewModelSerializer

			Dim toValueStory As IStoryViewModel

			Dim factory = ViewModelLocator.Instance.ObjectFactoryInstance

			For Each fromValue In stories
				toValueStory = factory.Create(Of IStoryViewModel)()

				serializer.Deserialize(fromValue, toValueStory)

				toValue.Stories.Add(toValueStory)
			Next fromValue
		End Sub

		Public Sub Serialize()
			If ViewModelLocator.Instance.StoryListInstance IsNot Nothing Then
				Dim stories As IStoryListViewModel = ViewModelLocator.Instance.StoryListInstance

				Serialize(stories)
			End If
		End Sub

		Private Sub Serialize(ByVal fromValue As IStoryListViewModel)
			Dim document = XmlUtility.StringToXDocument("<stories />")

			WriteToDocument(document, fromValue)

			SetStateValue(SERIALIZATION_KEY_STORY_LIST, document.ToString)
		End Sub

		Private Sub WriteToDocument(ByVal document As XDocument, ByVal storyList As IStoryListViewModel)
			Dim root = document.Root

			root.SetElementValue("Id", storyList.Id)
			root.SetElementValue("Title", storyList.Title)
			root.SetElementValue("UrlToHtml", storyList.UrlToHtml)

			Dim storySerializer = New StoryViewModelSerializer

			For Each fromValue In storyList.Stories
				root.Add(storySerializer.SerializeToElement(fromValue))
			Next fromValue
		End Sub
	End Class
End Namespace
