Imports Microsoft.Phone.Shell
Imports Benday.Npr.Presentation
Imports Benday.Npr.Common
Imports Benday.Npr.Presentation.Interfaces

Namespace Benday.Npr.Phone
	Public Class StoryViewModelSerializer
		Inherits ViewModelSerializerBase

		Public Sub New()
		End Sub

		Public Sub Serialize()
			If ViewModelLocator.Instance.CurrentStory IsNot Nothing Then
				Dim story = ViewModelLocator.Instance.CurrentStory

				Serialize(story)
			End If
		End Sub

		Public Function SerializeToElement(ByVal serializeThis As IStoryViewModel) As XElement
			Return WriteToElement(serializeThis, New XElement("story"))
		End Function

		Private Sub Serialize(ByVal story As Benday.Npr.Presentation.Interfaces.IStoryViewModel)
			Dim document = XmlUtility.StringToXDocument("<story />")

			WriteToDocument(document, story)

			SetStateValue(SERIALIZATION_KEY_STORY, document.ToString)
		End Sub

		Private Sub PopulateStoryFromDocument(ByVal document As XDocument, ByVal toValue As IStoryViewModel)
			Dim fromValue = document.Root

			Deserialize(fromValue, toValue)
		End Sub

		Private Sub WriteToDocument(ByVal document As XDocument, ByVal story As IStoryViewModel)
			Dim root = document.Root

			WriteToElement(story, root)
		End Sub

		Private Function WriteToElement(ByVal story As IStoryViewModel, ByVal toElement As XElement) As XElement
			XmlUtility.SetChildElement(toElement, "Id", story.Id)
			XmlUtility.SetChildElement(toElement, "UrlToHtml", story.UrlToHtml)
			XmlUtility.SetChildElement(toElement, "UrlToApi", story.UrlToApi)
			XmlUtility.SetChildElement(toElement, "UrlShort", story.UrlShort)
			XmlUtility.SetChildElement(toElement, "Title", story.Title)
			XmlUtility.SetChildElement(toElement, "Teaser", story.Teaser)
			XmlUtility.SetChildElement(toElement, "TeaserMini", story.TeaserMini)
			XmlUtility.SetChildElement(toElement, "Slug", story.Slug)
			XmlUtility.SetChildElement(toElement, "StoryDate", story.StoryDate)
			XmlUtility.SetChildElement(toElement, "PublishDate", story.PublishDate)
			XmlUtility.SetChildElement(toElement, "ShowName", story.ShowName)
			XmlUtility.SetChildElement(toElement, "ShowDate", story.ShowDate)
			XmlUtility.SetChildElement(toElement, "OrganizationName", story.OrganizationName)
			XmlUtility.SetChildElement(toElement, "OrganizationAbbreviation", story.OrganizationAbbreviation)
			XmlUtility.SetChildElement(toElement, "OrganizationUrl", story.OrganizationUrl)
			XmlUtility.SetChildElement(toElement, "Duration", story.Duration)
			XmlUtility.SetChildElement(toElement, "UrlToMp3", story.UrlToMp3)
			XmlUtility.SetChildElement(toElement, "UrlToWindowsMedia", story.UrlToWindowsMedia)
			XmlUtility.SetChildElement(toElement, "Author", story.Author)

			Dim paragraphs As New XElement("Paragraphs")

			toElement.Add(paragraphs)

			For Each currentParagraph In story.Paragraphs
				paragraphs.Add(New XElement("Paragraph", currentParagraph.Text))
			Next currentParagraph

			Return toElement
		End Function

		Public Sub Deserialize(ByVal fromValue As XElement, ByVal toValue As IStoryViewModel)
			toValue.Id = XmlUtility.GetChildElementValue(fromValue, "Id", String.Empty)
			toValue.UrlToHtml = XmlUtility.GetChildElementValue(fromValue, "UrlToHtml", String.Empty)
			toValue.UrlToApi = XmlUtility.GetChildElementValue(fromValue, "UrlToApi", String.Empty)
			toValue.UrlShort = XmlUtility.GetChildElementValue(fromValue, "UrlShort", String.Empty)
			toValue.Title = XmlUtility.GetChildElementValue(fromValue, "Title", String.Empty)
			toValue.Teaser = XmlUtility.GetChildElementValue(fromValue, "Teaser", String.Empty)
			toValue.TeaserMini = XmlUtility.GetChildElementValue(fromValue, "TeaserMini", String.Empty)
			toValue.Slug = XmlUtility.GetChildElementValue(fromValue, "Slug", String.Empty)
            toValue.StoryDate = XmlUtility.GetChildElementValue(fromValue, "StoryDate", CType(Nothing, Date))
            toValue.PublishDate = XmlUtility.GetChildElementValue(fromValue, "PublishDate", CType(Nothing, Date))
			toValue.ShowName = XmlUtility.GetChildElementValue(fromValue, "ShowName", String.Empty)
            toValue.ShowDate = XmlUtility.GetChildElementValue(fromValue, "ShowDate", CType(Nothing, Date))
			toValue.OrganizationName = XmlUtility.GetChildElementValue(fromValue, "OrganizationName", String.Empty)
			toValue.OrganizationAbbreviation = XmlUtility.GetChildElementValue(fromValue, "OrganizationAbbreviation", String.Empty)
			toValue.OrganizationUrl = XmlUtility.GetChildElementValue(fromValue, "OrganizationUrl", String.Empty)
            toValue.Duration = XmlUtility.GetChildElementValue(fromValue, "Duration", CType(Nothing, Integer))
			toValue.UrlToMp3 = XmlUtility.GetChildElementValue(fromValue, "UrlToMp3", String.Empty)
			toValue.UrlToWindowsMedia = XmlUtility.GetChildElementValue(fromValue, "UrlToWindowsMedia", String.Empty)
			toValue.Author = XmlUtility.GetChildElementValue(fromValue, "Author", String.Empty)

			Dim paragraphs As IEnumerable(Of XElement) = fromValue.Elements("Paragraphs")

			Dim toParagraph As IParagraphViewModel
			Dim objectFactoryInstance As IObjectFactory = ViewModelLocator.Instance.ObjectFactoryInstance

			For Each currentParagraph In paragraphs
				toParagraph = objectFactoryInstance.Create(Of IParagraphViewModel)()

				toParagraph.Text = currentParagraph.Value

				toValue.Paragraphs.Add(toParagraph)
			Next currentParagraph
		End Sub

		Public Sub Deserialize()
			If HasStateValue(ViewModelSerializerBase.SERIALIZATION_KEY_STORY) = False Then
				Throw New InvalidOperationException("Could not find value for serialized story xml.")
			Else
				Try
					Dim builder As New System.Text.StringBuilder

					Dim serializedStoryXml As String = GetStateValue(ViewModelSerializerBase.SERIALIZATION_KEY_STORY)

					If String.IsNullOrEmpty(serializedStoryXml) = True Then
						Throw New InvalidOperationException("Serialized story xml was null or empty.")
					End If

					Dim document = XmlUtility.StringToXDocument(serializedStoryXml)

					Dim story As IStoryViewModel = ViewModelLocator.Instance.ObjectFactoryInstance.Create(Of IStoryViewModel)()

					PopulateStoryFromDocument(document, story)

					ViewModelLocator.Instance.CurrentStory = story
				Catch e1 As Exception
					Throw
				End Try
			End If
		End Sub
	End Class
End Namespace
