Imports GalaSoft.MvvmLight
Imports System.Windows.Input
Imports System.Collections.ObjectModel
Imports Benday.Npr.Common
Imports Benday.Npr.Presentation.Interfaces
Imports Benday.Npr.Common.DomainModel

Namespace Benday.Npr.Presentation
	Public Class AllProgramsViewModel
		Inherits ObjectFactoryViewModel
		Implements IAllProgramsViewModel
        Private Const IsNetworkAvailablePropertyName As String = "IsNetworkAvailable"

        Private m_IsNetworkAvailable As Boolean
        Public Overloads Property IsNetworkAvailable As Boolean Implements IAllProgramsViewModel.IsNetworkAvailable
            Get
                Return m_IsNetworkAvailable
            End Get
            Set(ByVal value As Boolean)
                m_IsNetworkAvailable = value
                RaisePropertyChanged(IsNetworkAvailablePropertyName)
            End Set
        End Property

		Public Sub New(ByVal service As IFeedDomainService, ByVal factory As IObjectFactory)
			MyBase.New(factory)
			If service Is Nothing Then
				Throw New ArgumentNullException("service", "service is null.")
			End If

			m_Programs = New ObservableCollection(Of IProgramViewModel)

			m_ServiceInstance = service
		End Sub

		Private m_ServiceInstance As IFeedDomainService
		Private ReadOnly Property ServiceInstance As IFeedDomainService
			Get
				Return m_ServiceInstance
			End Get
		End Property

		Public Sub Load() Implements IAllProgramsViewModel.Load
			ServiceInstance.GetAllPrograms(New ReturnResult(Of IFeed)(AddressOf FeedReturned))
		End Sub

		Private Sub FeedReturned(ByVal result As ReturnResult(Of IFeed))
			If result.Error Is Nothing AndAlso result.Result IsNot Nothing Then
				Title = result.Result.Title

				PopulatePrograms(result.Result.FeedItems)
			End If
		End Sub

		Private Sub PopulatePrograms(ByVal feedItems As IList(Of IFeedItem))
			Me.Programs.Clear()

			Dim toValue As IProgramViewModel = Nothing

			For Each fromValue In feedItems
				toValue = ObjectFactoryInstance.Create(Of IProgramViewModel)()

				toValue.Title = fromValue.Title
				toValue.Id = fromValue.Id
				toValue.AdditionalInformation = fromValue.AdditionalInfo

				Me.Programs.Add(toValue)
			Next fromValue
		End Sub

		Private Const TitlePropertyName As String = "Title"

		Private m_Title As String
		Public Property Title As String Implements IAllProgramsViewModel.Title
			Get
				Return m_Title
			End Get
			Set(ByVal value As String)
				m_Title = value
				RaisePropertyChanged(TitlePropertyName)
			End Set
		End Property

		Private m_Programs As ObservableCollection(Of IProgramViewModel)
		Public Property Programs As ObservableCollection(Of IProgramViewModel) Implements IAllProgramsViewModel.Programs
			Get
				Return m_Programs
			End Get
			Set(ByVal value As ObservableCollection(Of IProgramViewModel))
				m_Programs = value
			End Set
		End Property

		Private Const SelectedItemPropertyName As String = "SelectedItem"

		Private m_SelectedItem As IProgramViewModel
		Public Property SelectedItem As IProgramViewModel Implements IAllProgramsViewModel.SelectedItem
			Get
				Return m_SelectedItem
			End Get
			Set(ByVal value As IProgramViewModel)
				If m_SelectedItem IsNot value Then
					m_SelectedItem = value

					ViewModelLocator.Instance.CurrentProgram = m_SelectedItem

					RaisePropertyChanged(SelectedItemPropertyName)

					RaiseEvent OnSelectionChanged(Me, New EventArgs)
				End If
			End Set
		End Property

		Public Event OnSelectionChanged As EventHandler Implements IAllProgramsViewModel.OnSelectionChanged

		Private m_MediaPlayback As IMediaPlaybackViewModel
		Public ReadOnly Property MediaPlayback As IMediaPlaybackViewModel Implements IAllProgramsViewModel.MediaPlayback
			Get
				If m_MediaPlayback Is Nothing Then
					m_MediaPlayback = ObjectFactoryInstance.Create(Of IMediaPlaybackViewModel)()
				End If

				Return m_MediaPlayback
			End Get
		End Property
	End Class
End Namespace
