Imports GalaSoft.MvvmLight
Imports System.Windows.Input
Imports System.Collections.ObjectModel
Imports Benday.Npr.Common
Imports Benday.Npr.Presentation.Interfaces
Imports Benday.Npr.Common.DomainModel
Imports System.Windows

Namespace Benday.Npr.Presentation
	Public Class MediaPlaybackViewModel
		Inherits NetworkAvailabilityViewModel
		Implements IMediaPlaybackViewModel

		Public Sub New()
			IsPlayable = True
		End Sub
        Public Overloads Property IsNetworkAvailable As Boolean Implements IMediaPlaybackViewModel.IsNetworkAvailable
		Public Function IsLightThemeEnabled() As Boolean
			Dim lightTheme As Visibility = CType(Application.Current.Resources("PhoneLightThemeVisibility"), Visibility)

            If lightTheme = System.Windows.Visibility.Visible Then
                Return True
            Else
                Return False
            End If
		End Function

		Private Const IsPlayablePropertyName As String = "IsPlayable"

		Private m_IsPlayable As Boolean
		Public Property IsPlayable As Boolean Implements IMediaPlaybackViewModel.IsPlayable
			Get
				Return m_IsPlayable
			End Get
			Set(ByVal value As Boolean)
				m_IsPlayable = value
				RaisePropertyChanged(IsPlayablePropertyName)
			End Set
		End Property

		Private Const UrlToMp3PropertyName As String = "UrlToMp3"

		Private m_UrlToMp3 As String
		Public Property UrlToMp3 As String Implements IMediaPlaybackViewModel.UrlToMp3
			Get
				Return m_UrlToMp3
			End Get
			Set(ByVal value As String)
				m_UrlToMp3 = value
				RaisePropertyChanged(UrlToMp3PropertyName)
			End Set
		End Property

		Private Const PlayPauseIconUriPropertyName As String = "PlayPauseIconUri"

		Private m_PlayPauseIconUri As String
		Public Property PlayPauseIconUri As String Implements IMediaPlaybackViewModel.PlayPauseIconUri
			Get
				If m_PlayPauseIconUri Is Nothing Then
					SetToPlay()
				End If

				Return m_PlayPauseIconUri
			End Get
			Set(ByVal value As String)
				m_PlayPauseIconUri = value
				RaisePropertyChanged(PlayPauseIconUriPropertyName)
			End Set
		End Property

		Private Const PlayPauseTextPropertyName As String = "PlayPauseText"

		Private m_PlayPauseText As String
		Public Property PlayPauseText As String Implements IMediaPlaybackViewModel.PlayPauseText
			Get
				If m_PlayPauseText Is Nothing Then
					m_PlayPauseText = "Play"
				End If

				Return m_PlayPauseText
			End Get
			Set(ByVal value As String)
				m_PlayPauseText = value
				RaisePropertyChanged(PlayPauseTextPropertyName)
			End Set
		End Property

		Private Sub SetToPlay()
			If IsLightThemeEnabled() = False Then
				PlayPauseIconUri = "/icons/appbar.transport.play.rest.png"
			Else
				PlayPauseIconUri = "/icons/light/appbar.transport.play.rest.png"
			End If

			PlayPauseText = "Play"
		End Sub

		Private Sub SetToPause()
			If IsLightThemeEnabled() = False Then
				PlayPauseIconUri = "/icons/appbar.transport.pause.rest.png"
			Else
				PlayPauseIconUri = "/icons/light/appbar.transport.pause.rest.png"
			End If

			PlayPauseText = "Pause"
		End Sub

		Public Sub PlayPause() Implements IMediaPlaybackViewModel.PlayPause
			If PlayPauseText = "Pause" Then
				SetToPause()
			Else
				SetToPlay()
			End If
		End Sub

		Public Sub SetState(ByVal state As String) Implements IMediaPlaybackViewModel.SetState
			If state = "Playing" Then
				SetToPause()
				IsPlayable = True
			ElseIf state = "Paused" Then
				SetToPlay()
				IsPlayable = True
			ElseIf state = "Stopped" Then
				SetToPlay()
				IsPlayable = False
			End If
		End Sub
	End Class
End Namespace
