Imports GalaSoft.MvvmLight
Imports System.Windows.Input
Imports System.Collections.ObjectModel
Imports Benday.Npr.Common
Imports Benday.Npr.Presentation.Interfaces
Imports Benday.Npr.Common.DomainModel
Imports System.Windows

Namespace Benday.Npr.Presentation
	Public Class StoryListViewModel
		Inherits ObjectFactoryViewModel
		Implements IStoryListViewModel
        Public Property IsNetworkAvaialbe As Boolean Implements IStoryListViewModel.IsNetworkAvailable
		Public Sub New(ByVal service As IStoryDomainService, ByVal factory As IObjectFactory)
			MyBase.New(factory)
			If service Is Nothing Then
				Throw New ArgumentNullException("service", "service is null.")
			End If

			m_Stories = New ObservableCollection(Of IStoryViewModel)

			m_ServiceInstance = service
		End Sub

		Private m_ServiceInstance As IStoryDomainService
		Private ReadOnly Property ServiceInstance As IStoryDomainService
			Get
				Return m_ServiceInstance
			End Get
		End Property

		Public Sub Load(ByVal programId As String) Implements IStoryListViewModel.Load
			IsProgressBarVisible = True
			ProgressBarMessage = "Loading stories..."

			Id = programId
			ServiceInstance.GetStories(New ReturnResult(Of IStoryList)(AddressOf StoryListReturned), programId)
		End Sub

		Private Sub StoryListReturned(ByVal result As ReturnResult(Of IStoryList))
			If result.Error Is Nothing Then
				Populate(result.Result)
			Else
				IsProgressBarVisible = False

				MessageBox.Show("Sorry.  We had some problems getting the stories.  (shrug)")
			End If
		End Sub

		Private Sub Populate(ByVal fromValues As IStoryList)
			ProgressBarMessage = "Parsing stories..."

			Try
				Me.Stories.Clear()

				If fromValues Is Nothing Then
					Title = String.Empty
					Teaser = String.Empty
					TeaserMini = String.Empty
					UrlToApi = String.Empty
					UrlToHtml = String.Empty

					MessageBox.Show("That's weird.  We didn't get anything back from NPR.  Any chance you just lost your internet connection?")
				Else
					Title = fromValues.Title
					Teaser = fromValues.Teaser
					TeaserMini = fromValues.TeaserMini
					UrlToApi = fromValues.UrlToApi
					UrlToHtml = fromValues.UrlToHtml

					Dim toValue As IStoryViewModel = Nothing

					For Each fromValue In fromValues.Stories
						toValue = ObjectFactoryInstance.Create(Of IStoryViewModel)()

						toValue.Load(fromValue)

						Me.Stories.Add(toValue)
					Next fromValue
				End If
			Finally
				IsProgressBarVisible = False
			End Try
		End Sub

		Private Const IdPropertyName As String = "Id"

		Private m_Id As String
		Public Property Id As String Implements IStoryListViewModel.Id
			Get
				Return m_Id
			End Get
			Set(ByVal value As String)
				m_Id = value
				RaisePropertyChanged(IdPropertyName)
			End Set
		End Property

		Private Const TitlePropertyName As String = "Title"

		Private m_Title As String
		Public Property Title As String Implements IStoryListViewModel.Title
			Get
				Return m_Title
			End Get
			Set(ByVal value As String)
				m_Title = value
				RaisePropertyChanged(TitlePropertyName)
			End Set
		End Property

		Private m_Stories As ObservableCollection(Of IStoryViewModel)
		Public ReadOnly Property Stories As ObservableCollection(Of IStoryViewModel) Implements IStoryListViewModel.Stories
			Get
				Return m_Stories
			End Get
		End Property

		Private Const TeaserPropertyName As String = "Teaser"

		Private m_Teaser As String
		Public Property Teaser As String
			Get
				Return m_Teaser
			End Get
			Set(ByVal value As String)
				m_Teaser = value
				RaisePropertyChanged(TeaserPropertyName)
			End Set
		End Property

		Private Const TeaserMiniPropertyName As String = "TeaserMini"

		Private m_TeaserMini As String
		Public Property TeaserMini As String
			Get
				Return m_TeaserMini
			End Get
			Set(ByVal value As String)
				m_TeaserMini = value
				RaisePropertyChanged(TeaserMiniPropertyName)
			End Set
		End Property

		Private Const UrlToHtmlPropertyName As String = "UrlToHtml"

		Private m_UrlToHtml As String
		Public Property UrlToHtml As String Implements IStoryListViewModel.UrlToHtml
			Get
				Return m_UrlToHtml
			End Get
			Set(ByVal value As String)
				m_UrlToHtml = value
				RaisePropertyChanged(UrlToHtmlPropertyName)
			End Set
		End Property

		Private Const UrlToApiPropertyName As String = "UrlToApi"

		Private m_UrlToApi As String
		Public Property UrlToApi As String
			Get
				Return m_UrlToApi
			End Get
			Set(ByVal value As String)
				m_UrlToApi = value
				RaisePropertyChanged(UrlToApiPropertyName)
			End Set
		End Property

		Private Const SelectedItemPropertyName As String = "SelectedItem"

		Private m_SelectedItem As IStoryViewModel
		Public Property SelectedItem As IStoryViewModel Implements IStoryListViewModel.SelectedItem
			Get
				Return m_SelectedItem
			End Get
			Set(ByVal value As IStoryViewModel)
				If m_SelectedItem IsNot value Then
					m_SelectedItem = value

					ViewModelLocator.Instance.CurrentStory = m_SelectedItem

					RaisePropertyChanged(SelectedItemPropertyName)

					RaiseEvent OnSelectionChanged(Me, New EventArgs)
				End If
			End Set
		End Property

		Public Event OnSelectionChanged As EventHandler Implements IStoryListViewModel.OnSelectionChanged

		Private Const IsProgressBarVisiblePropertyName As String = "IsProgressBarVisible"

		Private m_IsProgressBarVisible As Boolean
		Public Property IsProgressBarVisible As Boolean Implements Benday.Npr.Presentation.Interfaces.IProgressBarViewModel.IsProgressBarVisible
			Get
				Return m_IsProgressBarVisible
			End Get
			Set(ByVal value As Boolean)
				m_IsProgressBarVisible = value
				RaisePropertyChanged(IsProgressBarVisiblePropertyName)
			End Set
		End Property

		Private Const ProgressBarMessagePropertyName As String = "ProgressBarMessage"

		Private m_ProgressBarMessage As String
		Public Property ProgressBarMessage As String Implements Benday.Npr.Presentation.Interfaces.IProgressBarViewModel.ProgressBarMessage
			Get
				Return m_ProgressBarMessage
			End Get
			Set(ByVal value As String)
				m_ProgressBarMessage = value
				RaisePropertyChanged(ProgressBarMessagePropertyName)
			End Set
		End Property
	End Class
End Namespace
