'
'  In App.xaml:
'  <Application.Resources>
'      <vm:ViewModelLocator xmlns:vm="clr-namespace:Benday.Npr.Presentation"
'                                   x:Key="Locator" />
'  </Application.Resources>
'  
'  In the View:
'  DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
'

Imports Benday.Npr.Presentation.Interfaces
Imports Benday.Npr.Common
Imports GalaSoft.MvvmLight

Namespace Benday.Npr.Presentation
	''' <summary>
	''' This class contains static references to all the view models in the
	''' application and provides an entry point for the bindings.
	''' <para>
	''' Use the <strong>mvvmlocatorproperty</strong> snippet to add ViewModels
	''' to this locator.
	''' </para>
	''' <para>
	''' In Silverlight and WPF, place the ViewModelLocator in the App.xaml resources:
	''' </para>
	''' <code>
    ''' &lt;Application.Resources&gt
	'''     &lt;vm:ViewModelLocator xmlns:vm="clr-namespace:Benday.Npr.Presentation"
    '''                                  x:Key="Locator" /&gt
    ''' &lt;/Application.Resources&gt
	''' </code>
	''' <para>
	''' Then use:
	''' </para>
	''' <code>
	''' DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
	''' </code>
	''' <para>
	''' You can also use Blend to do all this with the tool's support.
	''' </para>
	''' <para>
	''' See http://www.galasoft.ch/mvvm/getstarted
	''' </para>
	''' </summary>
	Public Class ViewModelLocator
		Inherits ViewModelBase

		''' <summary>
		''' Initializes a new instance of the ViewModelLocator class.
		''' </summary>
		Private Sub New()
            ''  If ViewModelBase.IsInDesignModeStatic Then
            ''    ' Create design time view models
            ''Else
            ''    ' Create run time view models
            ''End If
        End Sub

		Public Shared Sub CreateInstance(ByVal factory As IObjectFactory)
			m_Instance = New ViewModelLocator(factory)
		End Sub

		Private Shared m_Instance As ViewModelLocator
		Public Shared Property Instance As ViewModelLocator
			Get
				If m_Instance Is Nothing Then
					Throw New InvalidOperationException("Not initialized.")
				End If

				Return m_Instance
			End Get
			Set(ByVal value As ViewModelLocator)
				m_Instance = value
			End Set
		End Property

		Public Sub New(ByVal factory As IObjectFactory)
			m_ObjectFactoryInstance = factory
		End Sub

		Private m_ObjectFactoryInstance As IObjectFactory
		Public ReadOnly Property ObjectFactoryInstance As IObjectFactory
			Get
				Return m_ObjectFactoryInstance
			End Get
		End Property

		Private m_AllPrograms As IAllProgramsViewModel
		Public ReadOnly Property AllPrograms As IAllProgramsViewModel
			Get
				If m_AllPrograms Is Nothing Then
					m_AllPrograms = ObjectFactoryInstance.Create(Of IAllProgramsViewModel)()
				End If

				Return m_AllPrograms
			End Get
		End Property

		Private Const CurrentProgramPropertyName As String = "CurrentProgram"

		Private m_CurrentProgram As IProgramViewModel
		Public Property CurrentProgram As IProgramViewModel
			Get
				Return m_CurrentProgram
			End Get
			Set(ByVal value As IProgramViewModel)
				m_CurrentProgram = value
				RaisePropertyChanged(CurrentProgramPropertyName)
			End Set
		End Property

		Private m_StoryListInstance As IStoryListViewModel
		Public ReadOnly Property StoryListInstance As IStoryListViewModel
			Get
				If m_StoryListInstance Is Nothing Then
					m_StoryListInstance = ObjectFactoryInstance.Create(Of IStoryListViewModel)()
				End If

				Return m_StoryListInstance
			End Get
		End Property

		Private Const CurrentStoryPropertyName As String = "CurrentStory"

		Private m_CurrentStory As IStoryViewModel
		Public Property CurrentStory As IStoryViewModel
			Get
				Return m_CurrentStory
			End Get
			Set(ByVal value As IStoryViewModel)
				m_CurrentStory = value
				RaisePropertyChanged(CurrentStoryPropertyName)
			End Set
		End Property

		Public Sub NotifyNetworkAvailabilityChange(ByVal isNetworkAvailable As Boolean)
			If Me.AllPrograms IsNot Nothing Then
				Me.AllPrograms.IsNetworkAvailable = isNetworkAvailable
			End If

			If Me.CurrentProgram IsNot Nothing Then
				Me.CurrentProgram.IsNetworkAvailable = isNetworkAvailable
			End If

			If Me.CurrentStory IsNot Nothing Then
				Me.CurrentStory.IsNetworkAvailable = isNetworkAvailable
			End If

			If Me.StoryListInstance IsNot Nothing Then
				Me.StoryListInstance.IsNetworkAvailable = isNetworkAvailable
			End If
		End Sub
	End Class
End Namespace
