Imports System.Text
Imports System.IO.IsolatedStorage
Imports System.IO

Namespace Benday.Npr.Phone.DataAccess
	Public NotInheritable Class DataAccessUtility
		Private Sub New()
		End Sub

		Public Shared Function ReplaceUndisplayableHtml(ByVal original As String) As String
			If original Is Nothing Then
				Return Nothing
			Else
				Dim cleaned As String = original

				cleaned = RemoveComments(cleaned)
				cleaned = cleaned.Replace("<br />", Environment.NewLine)

				Dim builder As New StringBuilder(cleaned)

				builder.Replace("&lt;em&gt;", String.Empty)
				builder.Replace("&lt;/em&gt;", String.Empty)
				builder.Replace("<em>", String.Empty)
				builder.Replace("</em>", String.Empty)
				builder.Replace("&lt;strong&gt;", String.Empty)
				builder.Replace("&lt;/strong&gt;", String.Empty)
				builder.Replace("<strong>", String.Empty)
				builder.Replace("</strong>", String.Empty)
				builder.Replace("</p>", String.Empty)
				builder.Replace("<p>", String.Empty)
				builder.Replace("&#160;", " ")
				builder.Replace("&#8217;", "'")

				cleaned = RemoveHtmlTags(builder.ToString)

				Return cleaned.Trim
			End If
		End Function

		Private Shared Function RemoveHtmlTags(ByVal contents As String) As String
			If contents.Contains("<") = True Then
				Dim startOfTag As Integer = contents.IndexOf("<")
				Dim endOfTag As Integer = contents.IndexOf(">", startOfTag)

				If startOfTag <> -1 AndAlso endOfTag <> -1 AndAlso (endOfTag - startOfTag + 1) < contents.Length Then
					contents = contents.Remove(startOfTag, (endOfTag - startOfTag + 1))

					contents = RemoveHtmlTags(contents)
				End If
			End If

			Return contents
		End Function

		Private Shared Function RemoveComments(ByVal contents As String) As String
			If contents.Contains("<!--") = True Then
				Dim startOfComment As Integer = contents.IndexOf("<!--")
				Dim endOfComment As Integer = contents.IndexOf("-->")

				If startOfComment <> -1 AndAlso endOfComment <> -1 AndAlso (endOfComment + 3) < contents.Length Then
					contents = contents.Remove(startOfComment, (endOfComment - startOfComment + 3))

					' remove any other comments
					contents = RemoveComments(contents)
				End If
			End If

			Return contents
		End Function

		Public Shared Sub SaveFile(ByVal filename As String, ByVal doc As XDocument)
			SaveFile(filename, doc.ToString)
		End Sub

		Public Shared Sub SaveFile(ByVal filename As String, ByVal contents As String)
			Using userIsolatedStorage = IsolatedStorageFile.GetUserStoreForApplication
				If userIsolatedStorage.FileExists(filename) = True Then
					userIsolatedStorage.DeleteFile(filename)
				End If

				Using fileStream = userIsolatedStorage.OpenFile(filename, System.IO.FileMode.OpenOrCreate)
					Using writer As New StreamWriter(fileStream)
						writer.Write(contents)
					End Using
				End Using
			End Using
		End Sub

		Public Shared Function ReadFile(ByVal filename As String) As String
			Using userIsolatedStorage = IsolatedStorageFile.GetUserStoreForApplication
				If userIsolatedStorage.FileExists(filename) = False Then
					Return Nothing
				Else
                    Dim returnValue As String = String.Empty


					Using fileStream = userIsolatedStorage.OpenFile(filename, System.IO.FileMode.Open)
						Using reader As New StreamReader(fileStream)
							returnValue = reader.ReadToEnd
						End Using
					End Using

					Return returnValue
				End If
			End Using
		End Function

		Public Shared Function GetProgramStoriesFilename(ByVal programId As String) As String
			If String.IsNullOrEmpty(programId) Then
				Throw New ArgumentException("programId is null or empty.", "programId")
			End If

			Return String.Format(DataAccessConstants.FilenameProgramStoriesList, programId)
		End Function
	End Class
End Namespace
