Imports Benday.Npr.Phone.Common.Repositories
Imports Benday.Npr.Phone.Common.Models
Imports Benday.Npr.Phone.Common
Imports System.IO.IsolatedStorage

Namespace Benday.Npr.Phone.DataAccess
	Public Class ProgramRepository
		Inherits RepositoryBase
		Implements IProgramRepository

		Public Sub New(ByVal factory As IObjectFactory, ByVal adapter As IProgramXmlAdapter)
			MyBase.New(factory)
			If adapter Is Nothing Then
				Throw New ArgumentNullException("adapter", "adapter is null.")
			End If

			m_AdapterInstance = adapter
		End Sub

		Private m_AdapterInstance As IProgramXmlAdapter

        Public Sub GetAll(ByVal action As ReturnResult(Of IList(Of IProgramInfo))) Implements IProgramRepository.GetAll
            If action Is Nothing Then
                Throw New ArgumentNullException("action", "action is null.")
            End If

            Dim returnValue As List(Of IProgramInfo) = GetAll()

            action.Notify(returnValue)
        End Sub

        Private Function GetAll() As List(Of IProgramInfo)
            Dim allProgramsXml = DataAccessResources.NprAllPrograms

            Dim returnValue As New List(Of IProgramInfo)

            m_AdapterInstance.Adapt(allProgramsXml, returnValue)

            Return returnValue
        End Function

        Public Sub GetById(ByVal action As ReturnResult(Of IProgramInfo), ByVal id As String) Implements IProgramRepository.GetById
            If action Is Nothing Then
                Throw New ArgumentNullException("action", "action is null.")
            End If

            action.Notify(GetById(id))
        End Sub

        Public Function GetById(ByVal id As String) As IProgramInfo Implements IProgramRepository.GetById
            Dim returnValue As IProgramInfo = Nothing

            Dim allPrograms = GetAll()

            If allPrograms IsNot Nothing Then
                returnValue = (
                    From temp In allPrograms
                    Where temp.Id = id
                    Select temp).FirstOrDefault
            End If

            Return returnValue
        End Function

        Public Sub SaveCurrentProgramId(ByVal currentProgramId As String) Implements IProgramRepository.SaveCurrentProgramId
            Dim doc = XmlUtility.StringToXDocument("<info />")

            If currentProgramId Is Nothing Then
                currentProgramId = String.Empty
            Else
                currentProgramId = currentProgramId.Trim
            End If

            XmlUtility.SetChildElement(doc.Root, "CurrentProgramId", currentProgramId)

            DataAccessUtility.SaveFile(DataAccessConstants.FilenameProgramInformation, doc)
        End Sub

        Public Function GetCurrentProgramId() As String Implements IProgramRepository.GetCurrentProgramId
            Dim contents = DataAccessUtility.ReadFile(DataAccessConstants.FilenameProgramInformation)

            If contents Is Nothing Then
                Return Nothing
            Else
                Dim doc = XmlUtility.StringToXDocument(contents)

                Dim returnValue As String = XmlUtility.GetChildElementValue(doc.Root, "CurrentProgramId", CType(Nothing, String))

                If returnValue Is Nothing OrElse returnValue = String.Empty Then
                    Return Nothing
                Else
                    Return returnValue
                End If
            End If
        End Function
	End Class
End Namespace
