Imports Benday.Npr.Phone.Common.Repositories
Imports Benday.Npr.Phone.Common.Models
Imports Benday.Npr.Phone.Common

Namespace Benday.Npr.Phone.DataAccess
	Public Class ProgramXmlAdapter
		Inherits AdapterBase
		Implements IProgramXmlAdapter

		Public Sub New(ByVal factory As IObjectFactory)
			MyBase.New(factory)
		End Sub

        Public Sub Adapt(ByVal fromValue As String, ByVal toValue As IList(Of IProgramInfo)) Implements IProgramXmlAdapter.Adapt
            If String.IsNullOrEmpty(fromValue) Then
                Throw New ArgumentException("fromValue is null or empty.", "fromValue")
            End If
            If toValue Is Nothing Then
                Throw New ArgumentNullException("toValue", "toValue is null.")
            End If

            Dim doc = StringToXDocument(fromValue)

            Dim root = doc.Root

            Dim items = root.Descendants("item")

            Dim toValueProgramInfo As IProgramInfo = Nothing

            For Each item In items
                toValueProgramInfo = ObjectFactoryInstance.Create(Of IProgramInfo)()

                Adapt(item, toValueProgramInfo)

                toValue.Add(toValueProgramInfo)
            Next item
        End Sub

        Public Sub Adapt(ByVal fromValue As XElement, ByVal toValue As IProgramInfo) Implements IProgramXmlAdapter.Adapt

            If fromValue Is Nothing Then
                Throw New ArgumentNullException("fromValue", "fromValue is null.")
            End If
            If toValue Is Nothing Then
                Throw New ArgumentNullException("toValue", "toValue is null.")
            End If

            toValue.Id = XmlUtility.GetAttributeValue(fromValue, "id", Nothing)
            toValue.Name = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "title", String.Empty))
            toValue.Description = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "additionalInfo", String.Empty))
        End Sub
	End Class
End Namespace
