Imports Benday.Npr.Phone.Common
Imports Benday.Npr.Phone.DataAccess
Imports Benday.Npr.Phone.Presentation
Imports Benday.Npr.Phone.Common.Repositories
Imports Benday.Npr.DataAccess

Namespace Benday.Npr.Phone.Factories
	Public Class ObjectFactory
		Implements IObjectFactory

		Private m_additionalConfiguration As ObjectFactoryConfiguration

		Public Sub New()
			m_additionalConfiguration = Nothing
		End Sub

		Public Sub New(ByVal configuration As ObjectFactoryConfiguration)
			If configuration Is Nothing Then
				Throw New ArgumentNullException("configuration", "Argument cannot be null.")
			End If

			m_additionalConfiguration = configuration

			InitializeContainer("ObjectFactory")
		End Sub

		Private Sub InitializeContainer(ByVal containerConfigName As String)
			If m_additionalConfiguration IsNot Nothing Then
				RegisterAdditionalConfiguration(m_additionalConfiguration)
			End If
		End Sub

		Private Sub RegisterAdditionalConfiguration(ByVal configuration As ObjectFactoryConfiguration)
			If configuration Is Nothing Then
				Throw New ArgumentNullException("configuration", "Configuration is null.")
			End If

			For Each key As Type In configuration.Instances.Keys
				RegisterInstance(key, configuration.Instances(key))
			Next key
		End Sub

		Private m_Instances As Dictionary(Of Type, Object)
		Public ReadOnly Property Instances As Dictionary(Of Type, Object)
			Get
				If m_Instances Is Nothing Then
					m_Instances = New Dictionary(Of Type, Object)
				End If

				Return m_Instances
			End Get
		End Property

        Private Sub RegisterInstance(ByVal interfaceType As Type, ByVal instance As Object)
            If Instances.ContainsKey(interfaceType) = True Then
                Instances.Remove(interfaceType)
            End If

            Instances.Add(interfaceType, instance)
        End Sub

        Public Function Create(Of T As Class)() As T Implements IObjectFactory.Create
            Dim interfaceType = GetType(T)

            If 1 <> 1 Then

            ElseIf interfaceType.Name = "IProgramRepository" Then
                Return TryCast(New ProgramRepository(Me, New ProgramXmlAdapter(Me)), T)
            ElseIf interfaceType.Name = "IProgramInfo" Then
                Return TryCast(New ProgramInfo, T)
            ElseIf interfaceType.Name = "IProgramsViewModel" Then
                Return TryCast(New ProgramsViewModel(Create(Of IProgramRepository)()), T)
            ElseIf interfaceType.Name = "IStoryListViewModel" Then
                Return TryCast(New StoryListViewModel(Create(Of IProgramRepository)(), Create(Of IStoryRepository)(), Me), T)
            ElseIf interfaceType.Name = "IStoryViewModel" Then
                Return TryCast(New StoryViewModel(Create(Of IProgramRepository)(), Create(Of IStoryRepository)(), Me), T)
            ElseIf interfaceType.Name = "IStory" Then
                Return TryCast(New Story, T)
            ElseIf interfaceType.Name = "IStoryXmlAdapter" Then
                Return TryCast(New StoryXmlAdapter(Me), T)
            ElseIf interfaceType.Name = "IParagraph" Then
                Return TryCast(New Paragraph, T)
            ElseIf interfaceType.Name = "IStoryList" Then
                Return TryCast(New StoryList, T)
            ElseIf interfaceType.Name = "IStoryRepository" Then
                Return TryCast(New StoryRepository(Me, Me.Create(Of IStoryXmlAdapter)()), T)
            ElseIf interfaceType.Name = "IRssFeedXmlAdapter" Then
                Return TryCast(New RssFeedXmlAdapter(Me), T)
            ElseIf interfaceType.Name = "IParagraphViewModel" Then
                Return TryCast(New ParagraphViewModel, T)

                'ElseIf interfaceType.Name = "IFeed" Then
                '          Return TryCast(New Feed, T)
                '      ElseIf interfaceType.Name = "IFeedItem" Then
                '          Return TryCast(New FeedItem, T)
                '      ElseIf interfaceType.Name = "IFeedDomainService" Then
                '          Return TryCast(GetFeedDomainService(), T)
                '      ElseIf interfaceType.Name = "IFeedXmlAdapter" Then
                '          Return TryCast(New FeedXmlAdapter(Me), T)
                '      ElseIf interfaceType.Name = "IAllProgramsViewModel" Then
                '          Return TryCast(New AllProgramsViewModel(GetFeedDomainService, Me), T)
                '      ElseIf interfaceType.Name = "IProgramViewModel" Then
                '          Return TryCast(New ProgramViewModel, T)
                '      ElseIf interfaceType.Name = "IFeedRepository" Then
                '          Return TryCast(New FeedRepository(Me), T)
                '      ElseIf interfaceType.Name = "IStoryDomainService" Then
                '          Return TryCast(GetStoryDomainService(), T)
                '      ElseIf interfaceType.Name = "IStoryListViewModel" Then
                '          Return TryCast(New StoryListViewModel(Me.Create(Of IStoryDomainService), Me), T)
                '      ElseIf interfaceType.Name = "IStoryViewModel" Then
                '          Return TryCast(New StoryViewModel(Create(Of IStoryDomainService), Me), T)
                '      ElseIf interfaceType.Name = "IMediaPlaybackViewModel" Then
                '          Return TryCast(New MediaPlaybackViewModel, T)
                '      ElseIf interfaceType.Name = "IRssFeedXmlAdapter" Then
                '          Return TryCast(New RssFeedXmlAdapter(Me), T)
                '      ElseIf interfaceType.Name = "IRssFeedItemList" Then
                '          Return TryCast(New RssFeedItemList, T)
                '      ElseIf interfaceType.Name = "IRssFeedItem" Then
                '          Return TryCast(New RssFeedItem, T)
                '      End If

            Else
                Throw New InvalidOperationException(String.Format("Type '{0}' is unsupported.", interfaceType.FullName))
            End If

        End Function

        Public Sub Reconfigure(ByVal configurationName As String) Implements IObjectFactory.Reconfigure
            InitializeContainer(configurationName)
        End Sub
	End Class
End Namespace
