Imports System.Text

Imports System.Collections.ObjectModel
Imports System.Windows.Input
Imports GalaSoft.MvvmLight
Imports Benday.Npr.Phone.Presentation.Interfaces
Imports Benday.Npr.Phone.Common

Namespace Benday.Npr.Phone.Presentation
	Public Class MultiSelectListViewModel
		Inherits SingleSelectListViewModel

		Protected Sub New()
			MyBase.New()
		End Sub

		Public Sub New(ByVal values As IList(Of ISelectableItem))
			MyBase.New(values)
		End Sub

		Private Const SelectedItemsPropertyName As String = "SelectedItems"
		Private m_selectedItems As IList(Of ISelectableItem)
		Public Property SelectedItems As IList(Of ISelectableItem)
			Get
				m_selectedItems = GetSelectedItems(Items)

				Return m_selectedItems
			End Get
			Set(ByVal value As IList(Of ISelectableItem))
				m_selectedItems = value
				RaisePropertyChanged(SelectedItemsPropertyName)
				RaisePropertyChanged(HasSelectedItemsPropertyName)
			End Set
		End Property

		Protected Const HasSelectedItemsPropertyName As String = "HasSelectedItems"

		Public Overridable ReadOnly Property HasSelectedItems As Boolean
			Get
				Dim items = SelectedItems

				If items IsNot Nothing AndAlso items.Count > 0 Then
					Return True
				Else
					Return False
				End If
			End Get
		End Property

		Private Function GetSelectedItems(ByVal values As ObservableCollection(Of ISelectableItem)) As IList(Of ISelectableItem)
            Dim selected = (
                From temp In values
                Where temp.IsSelected = True
                Select temp).ToList

			Return selected
		End Function

        Public Overloads Sub SelectByValue(ByVal values As IList(Of Integer))
            Dim valuesAsStrings = New List(Of String)

            For Each value In values
                valuesAsStrings.Add(value.ToString)
            Next value

            SelectByValue(valuesAsStrings)
        End Sub

        Public Overloads Sub SelectByValue(ByVal values As IList(Of String))
            For Each item In Items
                item.IsSelected = False
            Next item

            If values IsNot Nothing Then
                For Each item In values
                    SelectByValue(item)
                Next item
            End If
        End Sub

		Public Overrides Sub SelectByValue(ByVal value As String)
			Dim item = GetByValue(Items, value)

			If item IsNot Nothing Then
				item.IsSelected = True
			End If
		End Sub

		Protected Overrides Sub OnItemPropertyChanged(ByVal sender As Object, ByVal e As System.ComponentModel.PropertyChangedEventArgs)
			If TypeOf sender Is ISelectableItem AndAlso e.PropertyName = "IsSelected" Then
				' don't do the refresh logic that SingleSelectListViewModel does when an item changes it's IsSelected property
				' only notify that there's been a change

				RaisePropertyChanged(HasSelectedItemsPropertyName)
			End If
		End Sub
	End Class
End Namespace
