Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Ink
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports GalaSoft.MvvmLight
Imports Benday.Npr.Phone.Presentation.Interfaces
Imports Benday.Npr.Phone.Common.Repositories
Imports Benday.Npr.Phone.Common
Imports Benday.Npr.Phone.Common.Models

Namespace Benday.Npr.Phone.Presentation
	Public Class ProgramsViewModel
		Inherits NetworkAvailabilityViewModel
		Implements IProgramsViewModel

		Public Sub New(ByVal repository As IProgramRepository)
			If repository Is Nothing Then
				Throw New ArgumentNullException("repository", "repository is null.")
			End If

			m_RepositoryInstance = repository

			m_Programs = New SingleSelectListViewModel(New List(Of ISelectableItem))
			AddHandler m_Programs.OnItemSelected, AddressOf m_Programs_OnItemSelected
		End Sub

		Private m_RepositoryInstance As IProgramRepository
		Public ReadOnly Property RepositoryInstance As IProgramRepository
			Get
				Return m_RepositoryInstance
			End Get
		End Property

		Private Sub m_Programs_OnItemSelected(ByVal sender As Object, ByVal e As EventArgs)
			HandleProgramSelected()
		End Sub

		Private Sub HandleProgramSelected()
			If Programs.SelectedItem Is Nothing Then
				CurrentProgramId = Nothing
				RepositoryInstance.SaveCurrentProgramId(Nothing)
			Else
				CurrentProgramId = Programs.SelectedItem.Id
				RepositoryInstance.SaveCurrentProgramId(CurrentProgramId)
			End If
		End Sub

		Private Const CurrentProgramIdPropertyName As String = "CurrentProgramId"

		Private m_CurrentProgramId As String
		Public Property CurrentProgramId As String Implements IProgramsViewModel.CurrentProgramId
			Get
				Return m_CurrentProgramId
			End Get
			Set(ByVal value As String)
				m_CurrentProgramId = value
				RaisePropertyChanged(CurrentProgramIdPropertyName)
			End Set
		End Property

		Private Const CurrentProgramPropertyName As String = "CurrentProgram"

		Private Const ProgramsPropertyName As String = "Programs"

		Private m_Programs As SingleSelectListViewModel
		Public Property Programs As SingleSelectListViewModel Implements IProgramsViewModel.Programs
			Get
				Return m_Programs
			End Get
			Set(ByVal value As SingleSelectListViewModel)
				m_Programs = value
				RaisePropertyChanged(ProgramsPropertyName)
			End Set
		End Property

		Public Sub Load() Implements IProgramsViewModel.Load
			RepositoryInstance.GetAll(New ReturnResult(Of IList(Of IProgramInfo))(AddressOf LoadCompleted))
			CurrentProgramId = RepositoryInstance.GetCurrentProgramId
		End Sub

		Private Sub LoadCompleted(ByVal result As ReturnResult(Of IList(Of IProgramInfo)))
			Programs.Items.Clear()

			If result Is Nothing OrElse result.Error IsNot Nothing OrElse result.Result Is Nothing Then
				RequestMessageBox("No results.")
			Else
				Dim toValues = New List(Of ISelectableItem)
				Dim toValue As ISelectableItem = Nothing

				For Each fromValue In result.Result
					toValue = New SelectableItem(False, fromValue.Id, fromValue.Name, fromValue.Id, fromValue.Description)

					toValues.Add(toValue)
				Next fromValue

				Programs.Initialize(toValues)
			End If
		End Sub
	End Class
End Namespace
