Imports System.Collections.ObjectModel
Imports GalaSoft.MvvmLight
Imports System.Collections.Specialized
Imports System.ComponentModel
Imports Benday.Npr.Phone.Presentation.Interfaces
Imports Benday.Npr.Phone.Common

Namespace Benday.Npr.Phone.Presentation
	Public Class SelectableCollectionViewModel(Of T As {Class, ISelectable})
		Inherits ViewModelBase

		Public Sub New()
		End Sub

		Public Event OnItemSelected As EventHandler

		Private Sub RaiseOnItemSelected()
			RaiseEvent OnItemSelected(Me, New EventArgs)
		End Sub

		Public Sub New(ByVal values As ObservableCollection(Of T))
			If values Is Nothing Then
				Throw New ArgumentNullException("values", "values is null.")
			End If

			Items = values

			RefreshSelectedItem()
		End Sub

		Public Sub New(ByVal values As ObservableCollection(Of T), ByVal selectedItem As T)
			Me.New(values)
			If selectedItem Is Nothing Then
				Throw New ArgumentNullException("selectedItem", "selectedItem is null.")
			End If

			Items = values

			selectedItem.IsSelected = True

			RefreshSelectedItem()
		End Sub

		Public Overridable Sub Initialize(ByVal values As IEnumerable(Of T))
			Items = New ObservableCollection(Of T)(values)

			RefreshSelectedItem()
		End Sub

		Protected Sub RefreshSelectedItem()
			If Items Is Nothing Then
				' do nothing
			Else
                Dim selected = (
                    From temp In Items
                    Where temp.IsSelected = True
                    Select temp).FirstOrDefault

				Me.SelectedItem = selected
			End If
		End Sub

		Public Sub Clear()
			If Items IsNot Nothing Then
				Items.Clear()
			End If
		End Sub

		Public Sub Add(ByVal addThis As T)
			If Items Is Nothing Then
				Throw New InvalidOperationException("Items collection has not been initialized.")
			Else
				Items.Add(addThis)
			End If
		End Sub

		Public Sub Remove(ByVal removeThis As T)
			If Items Is Nothing Then
				Throw New InvalidOperationException("Items collection has not been initialized.")
			Else
				Items.Remove(removeThis)
			End If
		End Sub

		Private Const ItemsPropertyName As String = "Items"

		Protected m_Items As ObservableCollection(Of T)
		Public Property Items As ObservableCollection(Of T)
			Get
				If m_Items Is Nothing Then
					m_Items = New ObservableCollection(Of T)
				End If

				Return m_Items
			End Get
			Protected Set(ByVal value As ObservableCollection(Of T))
				m_Items = value

				SubscribeToINotifyPropertyChanged(m_Items)

				AddHandler m_Items.CollectionChanged, AddressOf m_items_CollectionChanged

				RaisePropertyChanged(ItemsPropertyName)
			End Set
		End Property

		Private Sub m_items_CollectionChanged(ByVal sender As Object, ByVal e As NotifyCollectionChangedEventArgs)
			SubscribeToINotifyPropertyChanged(e.NewItems)
		End Sub

		Private Sub SubscribeToINotifyPropertyChanged(ByVal items As System.Collections.IList)
			If items Is Nothing OrElse items.Count = 0 Then
				' do nothing
			Else
				For Each item In items
					If TypeOf item Is ISelectableItem Then
						SubscribeToINotifyPropertyChanged(TryCast(item, ISelectableItem))
					End If
				Next item
			End If
		End Sub

		Private Sub SubscribeToINotifyPropertyChanged(ByVal item As ISelectableItem)
			If item IsNot Nothing Then
				AddHandler item.PropertyChanged, AddressOf OnItemPropertyChanged
			End If
		End Sub

		Protected Overridable Sub OnItemPropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
			If TypeOf sender Is T AndAlso e.PropertyName = "IsSelected" Then
				Dim senderAsISelectableItem = TryCast(sender, T)

				If senderAsISelectableItem.IsSelected = True Then
					Me.SelectedItem = senderAsISelectableItem
				End If
			End If
		End Sub

		Private Function GetFirstSelectedItem(ByVal values As ObservableCollection(Of T)) As T
            Dim selected = (
                From temp In values
                Where temp.IsSelected = True
                Select temp).FirstOrDefault

			Return selected
		End Function

		Private Const SelectedItemPropertyName As String = "SelectedItem"
		Private m_SelectedItem As T
		Public Property SelectedItem As T
			Get
				m_SelectedItem = GetFirstSelectedItem(Items)

				Return m_SelectedItem
			End Get
			Set(ByVal value As T)
				If m_SelectedItem IsNot value Then
					DeselectCurrentItem()

					m_SelectedItem = value

					If m_SelectedItem IsNot Nothing Then
						m_SelectedItem.IsSelected = True
					End If

					RaiseOnItemSelected()
				End If

				RaisePropertyChanged(SelectedItemPropertyName)
			End Set
		End Property

		Private Sub DeselectCurrentItem()
			Dim currentSelected = GetFirstSelectedItem(Items)

			If currentSelected IsNot Nothing Then
				currentSelected.IsSelected = False
			End If
		End Sub
	End Class
End Namespace
