Imports System.Text
Imports System.Collections.ObjectModel
Imports System.Windows.Input
Imports GalaSoft.MvvmLight
Imports Benday.Npr.Phone.Presentation.Interfaces
Imports System.Collections.Specialized
Imports System.ComponentModel
Imports Benday.Npr.Phone.Common

Namespace Benday.Npr.Phone.Presentation
	Public Class SingleSelectListViewModel
		Inherits SelectableCollectionViewModel(Of ISelectableItem)

		Protected Sub New()
			MyBase.New()
			IsVisible = True
		End Sub

		Public Sub New(ByVal values As IEnumerable(Of ISelectableItem))
			Me.New(New ObservableCollection(Of ISelectableItem)(values))
			If values Is Nothing Then
				Throw New ArgumentNullException("values", "values is null.")
			End If

			IsVisible = True
		End Sub

		Public Sub New(ByVal values As IEnumerable(Of ISelectableItem), ByVal selectedItem As ISelectableItem)
			Me.New(New ObservableCollection(Of ISelectableItem)(values), selectedItem)
			If values Is Nothing Then
				Throw New ArgumentNullException("values", "values is null.")
			End If
			If selectedItem Is Nothing Then
				Throw New ArgumentNullException("selectedItem", "selectedItem is null.")
			End If

			IsVisible = True
		End Sub

		Public Sub New(ByVal values As ObservableCollection(Of ISelectableItem))
			MyBase.New(values)
			IsVisible = True
		End Sub

		Public Sub New(ByVal values As ObservableCollection(Of ISelectableItem), ByVal selectedItem As ISelectableItem)
			MyBase.New(values, selectedItem)
			IsVisible = True
		End Sub

		Public Overrides Sub Initialize(ByVal values As IEnumerable(Of ISelectableItem))
			Items = New ObservableCollection(Of ISelectableItem)(values)

			RefreshSelectedItem()

			IsVisible = True
		End Sub

		Public Sub SelectByText(ByVal text As String)
			SelectedItem = GetByText(Items, text)
		End Sub

		Private Function GetByText(ByVal values As ObservableCollection(Of ISelectableItem), ByVal text As String) As ISelectableItem
            Dim selected = (
                From temp In values
                Where temp.Text = text
                Select temp).FirstOrDefault

			Return selected
		End Function

		Protected Function GetByValue(ByVal values As ObservableCollection(Of ISelectableItem), ByVal value As String) As ISelectableItem
            Dim selected = (
                From temp In values
                Where temp.Value = value
                Select temp).FirstOrDefault

			Return selected
		End Function

		Public Overridable Sub SelectByValue(ByVal value As String)
			SelectedItem = GetByValue(Items, value)
		End Sub

		Public Sub SelectByValue(ByVal value As Integer)
			SelectedItem = GetByValue(Items, value.ToString)
		End Sub

		Private Const IsVisiblePropertyName As String = "IsVisible"

		Private m_IsVisible As Boolean
		Public Property IsVisible As Boolean
			Get
				Return m_IsVisible
			End Get
			Set(ByVal value As Boolean)
				m_IsVisible = value
				RaisePropertyChanged(IsVisiblePropertyName)
			End Set
		End Property

		Private Const IsValidPropertyName As String = "IsValid"

		Private m_IsValid As Boolean
		Public Property IsValid As Boolean
			Get
				Return m_IsValid
			End Get
			Set(ByVal value As Boolean)
				m_IsValid = value
				RaisePropertyChanged(IsValidPropertyName)
			End Set
		End Property

		Private Const ValidationMessagePropertyName As String = "ValidationMessage"

		Private m_ValidationMessage As String
		Public Property ValidationMessage As String
			Get
				Return m_ValidationMessage
			End Get
			Set(ByVal value As String)
				m_ValidationMessage = value
				RaisePropertyChanged(ValidationMessagePropertyName)
			End Set
		End Property
	End Class
End Namespace
