Imports Benday.Npr.Phone.Presentation.Interfaces
Imports Benday.Npr.Phone.Common.Models
Imports Benday.Npr.Phone.Common
Imports Benday.Npr.Phone.Common.Repositories
Imports System.Collections.ObjectModel

Namespace Benday.Npr.Phone.Presentation
	Public Class StoryViewModel
		Inherits ObjectFactoryViewModel
		Implements IStoryViewModel

		Public Sub New(ByVal programRepositoryInstance As IProgramRepository, ByVal storyRepositoryInstance As IStoryRepository, ByVal factory As IObjectFactory)
			MyBase.New(factory)
			If programRepositoryInstance Is Nothing Then
				Throw New ArgumentNullException("programRepositoryInstance", "programRepositoryInstance is null.")
			End If
			If storyRepositoryInstance Is Nothing Then
				Throw New ArgumentNullException("storyRepositoryInstance", "storyRepositoryInstance is null.")
			End If

			m_ProgramRepositoryInstance = programRepositoryInstance
			m_StoryRepositoryInstance = storyRepositoryInstance

			m_Paragraphs = New ObservableCollection(Of IParagraphViewModel)
		End Sub

		Private m_ProgramRepositoryInstance As IProgramRepository
		Private ReadOnly Property ProgramRepositoryInstance As IProgramRepository
			Get
				Return m_ProgramRepositoryInstance
			End Get
		End Property

		Private m_StoryRepositoryInstance As IStoryRepository
		Private ReadOnly Property StoryRepositoryInstance As IStoryRepository
			Get
				Return m_StoryRepositoryInstance
			End Get
		End Property

		Public Sub Load() Implements IStoryViewModel.Load
			CurrentProgramId = ProgramRepositoryInstance.GetCurrentProgramId
			Dim currentStoryId = StoryRepositoryInstance.GetCurrentStoryId

			Dim instance As IStory = StoryRepositoryInstance.GetStory(CurrentProgramId, currentStoryId)

			If instance Is Nothing Then
				Throw New ArgumentNullException("instance", "instance is null.")
			End If

			PopulateFromModel(instance)
		End Sub

		Private Sub PopulateFromModel(ByVal fromValue As IStory)
			Me.Id = fromValue.Id
			Me.UrlToHtml = fromValue.UrlToHtml
			Me.UrlToApi = fromValue.UrlToApi
			Me.UrlShort = fromValue.UrlShort
			Me.Title = fromValue.Title
			Me.Teaser = fromValue.Teaser
			Me.TeaserMini = fromValue.TeaserMini
			Me.Slug = fromValue.Slug
			Me.StoryDate = fromValue.StoryDate
			Me.PublishDate = fromValue.PublishDate
			Me.ShowName = fromValue.ShowName
			Me.ShowDate = fromValue.ShowDate
			Me.OrganizationName = fromValue.OrganizationName
			Me.OrganizationAbbreviation = fromValue.OrganizationAbbreviation
			Me.OrganizationUrl = fromValue.OrganizationUrl
			Me.Duration = fromValue.Duration
			Me.UrlToMp3 = fromValue.UrlToMp3
			Me.UrlToWindowsMedia = fromValue.UrlToWindowsMedia
			Me.Author = fromValue.Author

			m_Paragraphs.Clear()

			Dim toParagraph As IParagraphViewModel

			For Each fromParagraph In fromValue.Paragraphs
				toParagraph = ObjectFactoryInstance.Create(Of IParagraphViewModel)()

				toParagraph.Text = fromParagraph.Text

				Me.Paragraphs.Add(toParagraph)
			Next fromParagraph
		End Sub

		Private Const IdPropertyName As String = "Id"

		Private m_Id As String
		Public Property Id As String Implements IStoryViewModel.Id
			Get
				Return m_Id
			End Get
			Set(ByVal value As String)
				m_Id = value
				RaisePropertyChanged(IdPropertyName)
			End Set
		End Property

		Private Const UrlToHtmlPropertyName As String = "UrlToHtml"

		Private m_UrlToHtml As String
		Public Property UrlToHtml As String Implements IStoryViewModel.UrlToHtml
			Get
				Return m_UrlToHtml
			End Get
			Set(ByVal value As String)
				m_UrlToHtml = value
				RaisePropertyChanged(UrlToHtmlPropertyName)
			End Set
		End Property

		Private Const UrlToApiPropertyName As String = "UrlToApi"

		Private m_UrlToApi As String
		Public Property UrlToApi As String Implements IStoryViewModel.UrlToApi
			Get
				Return m_UrlToApi
			End Get
			Set(ByVal value As String)
				m_UrlToApi = value
				RaisePropertyChanged(UrlToApiPropertyName)
			End Set
		End Property

		Private Const UrlShortPropertyName As String = "UrlShort"

		Private m_UrlShort As String
		Public Property UrlShort As String Implements IStoryViewModel.UrlShort
			Get
				Return m_UrlShort
			End Get
			Set(ByVal value As String)
				m_UrlShort = value
				RaisePropertyChanged(UrlShortPropertyName)
			End Set
		End Property

		Private Const TitlePropertyName As String = "Title"

		Private m_Title As String
		Public Property Title As String Implements IStoryViewModel.Title
			Get
				Return m_Title
			End Get
			Set(ByVal value As String)
				m_Title = value
				RaisePropertyChanged(TitlePropertyName)
			End Set
		End Property

		Private Const TeaserPropertyName As String = "Teaser"

		Private m_Teaser As String
		Public Property Teaser As String Implements IStoryViewModel.Teaser
			Get
				Return m_Teaser
			End Get
			Set(ByVal value As String)
				m_Teaser = value
				RaisePropertyChanged(TeaserPropertyName)
			End Set
		End Property

		Private Const TeaserMiniPropertyName As String = "TeaserMini"

		Private m_TeaserMini As String
		Public Property TeaserMini As String Implements IStoryViewModel.TeaserMini
			Get
				Return m_TeaserMini
			End Get
			Set(ByVal value As String)
				m_TeaserMini = value
				RaisePropertyChanged(TeaserMiniPropertyName)
			End Set
		End Property

		Private Const SlugPropertyName As String = "Slug"

		Private m_Slug As String
		Public Property Slug As String Implements IStoryViewModel.Slug
			Get
				Return m_Slug
			End Get
			Set(ByVal value As String)
				m_Slug = value
				RaisePropertyChanged(SlugPropertyName)
			End Set
		End Property

		Private Const StoryDatePropertyName As String = "StoryDate"

		Private m_StoryDate As Date
		Public Property StoryDate As Date Implements IStoryViewModel.StoryDate
			Get
				Return m_StoryDate
			End Get
			Set(ByVal value As Date)
				m_StoryDate = value
				RaisePropertyChanged(StoryDatePropertyName)
			End Set
		End Property

		Private Const PublishDatePropertyName As String = "PublishDate"

		Private m_PublishDate As Date
		Public Property PublishDate As Date Implements IStoryViewModel.PublishDate
			Get
				Return m_PublishDate
			End Get
			Set(ByVal value As Date)
				m_PublishDate = value
				RaisePropertyChanged(PublishDatePropertyName)
			End Set
		End Property

		Private Const ShowNamePropertyName As String = "ShowName"

		Private m_ShowName As String
		Public Property ShowName As String Implements IStoryViewModel.ShowName
			Get
				Return m_ShowName
			End Get
			Set(ByVal value As String)
				m_ShowName = value
				RaisePropertyChanged(ShowNamePropertyName)
			End Set
		End Property

		Private Const ShowDatePropertyName As String = "ShowDate"

		Private m_ShowDate As Date
		Public Property ShowDate As Date Implements IStoryViewModel.ShowDate
			Get
				Return m_ShowDate
			End Get
			Set(ByVal value As Date)
				m_ShowDate = value
				RaisePropertyChanged(ShowDatePropertyName)
			End Set
		End Property

		Private Const OrganizationNamePropertyName As String = "OrganizationName"

		Private m_OrganizationName As String
		Public Property OrganizationName As String Implements IStoryViewModel.OrganizationName
			Get
				Return m_OrganizationName
			End Get
			Set(ByVal value As String)
				m_OrganizationName = value
				RaisePropertyChanged(OrganizationNamePropertyName)
			End Set
		End Property

		Private Const OrganizationAbbreviationPropertyName As String = "OrganizationAbbreviation"

		Private m_OrganizationAbbreviation As String
		Public Property OrganizationAbbreviation As String Implements IStoryViewModel.OrganizationAbbreviation
			Get
				Return m_OrganizationAbbreviation
			End Get
			Set(ByVal value As String)
				m_OrganizationAbbreviation = value
				RaisePropertyChanged(OrganizationAbbreviationPropertyName)
			End Set
		End Property

		Private Const OrganizationUrlPropertyName As String = "OrganizationUrl"

		Private m_OrganizationUrl As String
		Public Property OrganizationUrl As String Implements IStoryViewModel.OrganizationUrl
			Get
				Return m_OrganizationUrl
			End Get
			Set(ByVal value As String)
				m_OrganizationUrl = value
				RaisePropertyChanged(OrganizationUrlPropertyName)
			End Set
		End Property

		Private Const DurationPropertyName As String = "Duration"

		Private m_Duration As Integer
		Public Property Duration As Integer Implements IStoryViewModel.Duration
			Get
				Return m_Duration
			End Get
			Set(ByVal value As Integer)
				m_Duration = value
				RaisePropertyChanged(DurationPropertyName)
			End Set
		End Property

		Private Const HasUrlToMp3PropertyName As String = "HasUrlToMp3"

		Public Property HasUrlToMp3 As Boolean Implements IStoryViewModel.HasUrlToMp3
			Get
				If String.IsNullOrEmpty(UrlToMp3) = False Then
					Return True
				Else
					Return False
				End If
			End Get
			Set(ByVal value As Boolean)
				RaisePropertyChanged(HasUrlToMp3PropertyName)
			End Set
		End Property

		Private Const UrlToMp3PropertyName As String = "UrlToMp3"

		Private m_UrlToMp3 As String
		Public Property UrlToMp3 As String Implements IStoryViewModel.UrlToMp3
			Get
				Return m_UrlToMp3
			End Get
			Set(ByVal value As String)
				m_UrlToMp3 = value
				RaisePropertyChanged(UrlToMp3PropertyName)
				RaisePropertyChanged(HasUrlToMp3PropertyName)
			End Set
		End Property

		Private Const UrlToWindowsMediaPropertyName As String = "UrlToWindowsMedia"

		Private m_UrlToWindowsMedia As String
		Public Property UrlToWindowsMedia As String Implements IStoryViewModel.UrlToWindowsMedia
			Get
				Return m_UrlToWindowsMedia
			End Get
			Set(ByVal value As String)
				m_UrlToWindowsMedia = value
				RaisePropertyChanged(UrlToWindowsMediaPropertyName)
				RaisePropertyChanged(HasUrlToWindowsMediaPropertyName)
			End Set
		End Property

		Private Const HasUrlToWindowsMediaPropertyName As String = "HasUrlToWindowsMedia"

		Public Property HasUrlToWindowsMedia As Boolean Implements IStoryViewModel.HasUrlToWindowsMedia
			Get
				If String.IsNullOrEmpty(UrlToWindowsMedia) = False Then
					Return True
				Else
					Return False
				End If
			End Get
			Set(ByVal value As Boolean)
				RaisePropertyChanged(HasUrlToWindowsMediaPropertyName)
			End Set
		End Property

		Private Const AuthorPropertyName As String = "Author"

		Private m_Author As String
		Public Property Author As String Implements IStoryViewModel.Author
			Get
				Return m_Author
			End Get
			Set(ByVal value As String)
				m_Author = value
				RaisePropertyChanged(AuthorPropertyName)
			End Set
		End Property

		Private Const ParagraphsPropertyName As String = "Paragraphs"

		Private m_Paragraphs As ObservableCollection(Of IParagraphViewModel)
		Public Property Paragraphs As ObservableCollection(Of IParagraphViewModel) Implements IStoryViewModel.Paragraphs
			Get
				Return m_Paragraphs
			End Get
			Set(ByVal value As ObservableCollection(Of IParagraphViewModel))
				m_Paragraphs = value
				RaisePropertyChanged(ParagraphsPropertyName)
			End Set
		End Property

		Public Sub PlayMp3() Implements IStoryViewModel.PlayMp3
			If Me.HasUrlToMp3 = True Then
				If DoesUrlNeedToBeResolved() = True Then
					StoryRepositoryInstance.ResolveMp3UrlFromM3u(New ReturnResult(Of String)(AddressOf UrlResolved), Me.UrlToMp3)
				Else
					RaiseOnReadyToPlayMp3()
				End If
			End If
		End Sub

		Private Function DoesUrlNeedToBeResolved() As Boolean
			If HasResolvedUrlToMp3 = True Then
				Return False
			Else
				If UrlToMp3 IsNot Nothing AndAlso UrlToMp3.EndsWith(".mp3") = True Then
					ResolvedUrlToMp3 = UrlToMp3
					HasResolvedUrlToMp3 = True

					Return False
				Else
					Return True
				End If
			End If
		End Function

		Private Sub RaiseOnReadyToPlayMp3()
			RaiseEvent OnReadyToPlayMp3(Me, New EventArgs)
		End Sub

		Private Sub UrlResolved(ByVal action As ReturnResult(Of String))
			ResolvedUrlToMp3 = action.Result

			If HasResolvedUrlToMp3 = True Then
				RaiseOnReadyToPlayMp3()
			End If
		End Sub

		Private Const HasResolvedUrlToMp3PropertyName As String = "HasResolvedUrlToMp3"

		Public Property HasResolvedUrlToMp3 As Boolean Implements IStoryViewModel.HasResolvedUrlToMp3
			Get
				If String.IsNullOrEmpty(ResolvedUrlToMp3) = False Then
					Return True
				Else
					Return False
				End If
			End Get
			Set(ByVal value As Boolean)
				RaisePropertyChanged(HasResolvedUrlToMp3PropertyName)
			End Set
		End Property

		Private Const ResolvedUrlToMp3PropertyName As String = "ResolvedUrlToMp3"

		Private m_ResolvedUrlToMp3 As String
		Public Property ResolvedUrlToMp3 As String Implements IStoryViewModel.ResolvedUrlToMp3
			Get
				Return m_ResolvedUrlToMp3
			End Get
			Set(ByVal value As String)
				m_ResolvedUrlToMp3 = value
				RaisePropertyChanged(HasResolvedUrlToMp3PropertyName)
				RaisePropertyChanged(ResolvedUrlToMp3PropertyName)
			End Set
		End Property

		Public Event OnReadyToPlayMp3 As EventHandler Implements IStoryViewModel.OnReadyToPlayMp3

		Private Const CurrentProgramIdPropertyName As String = "CurrentProgramId"

		Private m_CurrentProgramId As String
        Public Property CurrentProgramId As String Implements IStoryViewModel.CurrentProgramId
            Get
                Return m_CurrentProgramId
            End Get
            Set(ByVal value As String)
                m_CurrentProgramId = value
                RaisePropertyChanged(CurrentProgramIdPropertyName)
            End Set
        End Property
      

		Public ReadOnly Property AudioTrackLocatorTag As String Implements IStoryViewModel.AudioTrackLocatorTag
			Get
				Return String.Format("nprlistener:{0}:{1}", CurrentProgramId, Id)
			End Get
		End Property
	End Class
End Namespace
