Imports GalaSoft.MvvmLight
Imports Benday.Npr.Phone.Common
Imports Benday.Npr.Phone.Presentation.Interfaces

Namespace Benday.Npr.Phone.Presentation
	Public Class ViewModelLocator
		Inherits ViewModelBase

		''' <summary>
		''' Initializes a new instance of the ViewModelLocator class.
		''' </summary>
		Private Sub New()
            '//if ViewModelBase.IsInDesignModeStatic then
            '//
            '//   Create design time view models
            '//
			'//else
            '//
			'//    // Create run time view models
            '//
		End Sub

		Public Shared Sub CreateInstance(ByVal factory As IObjectFactory)
			m_Instance = New ViewModelLocator(factory)
			m_Instance.NotifyNetworkAvailabilityChange(True)
		End Sub

		Public Shared Sub DestroyInstance()
			m_Instance = Nothing
		End Sub

		Private m_Story As IStoryViewModel
		Private m_StoryList As IStoryListViewModel
		Private Shared m_Instance As ViewModelLocator
		Public Shared Property Instance As ViewModelLocator
			Get
				If m_Instance Is Nothing Then
					Throw New InvalidOperationException("Not initialized.")
				End If

				Return m_Instance
			End Get
			Set(ByVal value As ViewModelLocator)
				m_Instance = value
			End Set
		End Property

		Public Sub New(ByVal factory As IObjectFactory)
			m_ObjectFactoryInstance = factory
		End Sub

		Private m_ObjectFactoryInstance As IObjectFactory
		Public ReadOnly Property ObjectFactoryInstance As IObjectFactory
			Get
				Return m_ObjectFactoryInstance
			End Get
		End Property

		Private m_Programs As IProgramsViewModel
		Public ReadOnly Property Programs As IProgramsViewModel
			Get
				If m_Programs Is Nothing Then
					m_Programs = ObjectFactoryInstance.Create(Of IProgramsViewModel)()
				End If

				Return m_Programs
			End Get
		End Property

		Public Property StoryList As IStoryListViewModel
			Get
				If m_StoryList Is Nothing Then
					m_StoryList = ObjectFactoryInstance.Create(Of IStoryListViewModel)()
				End If

				Return m_StoryList
			End Get
			Set(ByVal value As IStoryListViewModel)
				m_StoryList = value
			End Set
		End Property

		Public Property Story As IStoryViewModel
			Get
				If m_Story Is Nothing Then
					m_Story = ObjectFactoryInstance.Create(Of IStoryViewModel)()
				End If
				Return m_Story
			End Get
			Set(ByVal value As IStoryViewModel)
				m_Story = value
			End Set
		End Property

		Public Sub NotifyNetworkAvailabilityChange(ByVal isNetworkAvailable As Boolean)
			Programs.IsNetworkAvailable = True
			StoryList.IsNetworkAvailable = True
			Story.IsNetworkAvailable = True
		End Sub

		Private Const HasHandledNavigateFromPlaybackAgentPropertyName As String = "HasHandledNavigateFromPlaybackAgent"

		Private m_HasHandledNavigateFromPlaybackAgent As Boolean
		Public Property HasHandledNavigateFromPlaybackAgent As Boolean
			Get
				Return m_HasHandledNavigateFromPlaybackAgent
			End Get
			Set(ByVal value As Boolean)
				m_HasHandledNavigateFromPlaybackAgent = value
				RaisePropertyChanged(HasHandledNavigateFromPlaybackAgentPropertyName)
			End Set
		End Property
	End Class
End Namespace
