Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Microsoft.Silverlight.Testing
Imports Benday.Npr.Phone.DataAccess

Namespace Benday.Npr.Phone.UnitTests.DataAccess
	<TestClass, Tag("UndisplayableHtml")>
	Public Class DataAccessUtilityFixture
		Public Sub New()
		End Sub

		<TestMethod>
		Public Sub ReplaceUndisplayableHtml_NoUndisplayableHtmlReturnsOriginal()
			Dim checkThisString As String = "clean string"

			Dim actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString)

			Assert.AreEqual(Of String)(checkThisString, actual, "Should be the same.")

			AssertDoesNotContainUndisplayableHtml(actual)
		End Sub

		<TestMethod>
		Public Sub ReplaceUndisplayableHtml_IfOriginalIsNullReturnsNull()
            Dim checkThisString As String = String.Empty

			Dim actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString)

			Assert.AreEqual(Of String)(checkThisString, actual, "Should be the same.")
		End Sub

		<TestMethod>
		Public Sub ReplaceUndisplayableHtml_EscapedEmphasisTag_WithContent()
			Dim checkThisString As String = "This is some text &lt;em&gt;asdf&lt;/em&gt; more text."

			Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString))

			Dim actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString)

			AssertDoesNotContainUndisplayableHtml(actual)

			Assert.AreEqual(Of String)("This is some text asdf more text.", actual, "Should be the same.")
		End Sub

		<TestMethod>
		Public Sub ReplaceUndisplayableHtml_EscapedEmphasisTag_WithoutContent()
			Dim checkThisString As String = "This is some text &lt;em&gt;&lt;/em&gt; more text."

			Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString))

			Dim actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString)

			AssertDoesNotContainUndisplayableHtml(actual)

			Assert.AreEqual(Of String)("This is some text  more text.", actual, "Should be the same.")
		End Sub

		<TestMethod>
		Public Sub ReplaceUndisplayableHtml_UnescapedEmphasisTag_WithContent()
			Dim checkThisString As String = "This is some text <em>asdf</em> more text."

			Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString))

			Dim actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString)

			AssertDoesNotContainUndisplayableHtml(actual)

			Assert.AreEqual(Of String)("This is some text asdf more text.", actual, "Should be the same.")
		End Sub

		<TestMethod>
		Public Sub ReplaceUndisplayableHtml_UnescapedEmphasisTag_WithoutContent()
			Dim checkThisString As String = "This is some text <em></em> more text."

			Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString))

			Dim actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString)

			AssertDoesNotContainUndisplayableHtml(actual)

			Assert.AreEqual(Of String)("This is some text  more text.", actual, "Should be the same.")
		End Sub

		<TestMethod>
		Public Sub ReplaceUndisplayableHtml_UnescapedStrongTag_WithContent()
			Dim checkThisString As String = "This is some text <strong>asdf</strong> more text."

			Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString))

			Dim actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString)

			AssertDoesNotContainUndisplayableHtml(actual)

			Assert.AreEqual(Of String)("This is some text asdf more text.", actual, "Should be the same.")
		End Sub

		<TestMethod>
		Public Sub ReplaceUndisplayableHtml_UnescapedStrongTag_WithoutContent()
			Dim checkThisString As String = "This is some text <strong></strong> more text."

			Assert.IsTrue(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(checkThisString))

			Dim actual = DataAccessUtility.ReplaceUndisplayableHtml(checkThisString)

			AssertDoesNotContainUndisplayableHtml(actual)

			Assert.AreEqual(Of String)("This is some text  more text.", actual, "Should be the same.")
		End Sub

		Private Sub AssertDoesNotContainUndisplayableHtml(ByVal actual As String)
			Assert.IsFalse(UnitTestUtility.StringContainsUndisplayableHtmlFormatting(actual), "Should not contain any undisplayable html.  ('{0}')", actual)
		End Sub
	End Class
End Namespace
