Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Microsoft.Silverlight.Testing
Imports Benday.Npr.Phone.Common.Repositories
Imports Benday.Npr.Phone.DataAccess
Imports Benday.Npr.Phone.Common
Imports Benday.Npr.Phone.Common.Models

Namespace Benday.Npr.Phone.UnitTests.DataAccess
	<TestClass, Tag("Repository")>
	Public Class ProgramRepositoryFixture
		Inherits UnitTestBase

		Private m_WasNotifyCalled As Boolean = False
		Private m_DataReturnedFromGetAllCall As IList(Of IProgramInfo) = Nothing
		Private m_DataReturnedFromGetByIdCall As IProgramInfo = Nothing

		Protected Overrides Sub OnTestInitialize()
			UnitTestUtility.WipeProgramDataInIsolatedStorage()

			m_RepositoryInstance = Nothing
			m_WasNotifyCalled = False
			m_DataReturnedFromGetAllCall = Nothing
			m_DataReturnedFromGetByIdCall = Nothing
		End Sub

		Private m_RepositoryInstance As ProgramRepository
		Public ReadOnly Property RepositoryInstance As ProgramRepository
			Get
				If m_RepositoryInstance Is Nothing Then
					m_RepositoryInstance = TryCast(ObjectFactoryInstance.Create(Of IProgramRepository)(), ProgramRepository)
				End If

				Return m_RepositoryInstance
			End Get
		End Property

		<TestMethod>
		Public Sub GetAnInstanceOfIProgramRepositoryReturnsNonNullAndTypeofProgramRepository()
			Dim instance = ObjectFactoryInstance.Create(Of IProgramRepository)()

			Assert.IsNotNull(instance, "Instance of IProgramRepository returned from ObjectFactory was null.")
			Assert.IsInstanceOfType(instance, GetType(ProgramRepository), "Got the wrong type back.")
		End Sub

		<TestMethod>
		Public Sub RepositoryInstancePropertyOnTestInitializesToNonNull()
			Assert.IsNotNull(RepositoryInstance, "Instance was null.")
		End Sub

		<TestMethod>
		Public Sub GetAll()
			RepositoryInstance.GetAll(New ReturnResult(Of IList(Of IProgramInfo))(AddressOf GetAllHandler))

			Assert.IsTrue(m_WasNotifyCalled, "Notify wasn't called.")

			Assert.AreNotEqual(Of Integer)(0, m_DataReturnedFromGetAllCall.Count, "Result count was 0.")
		End Sub

		Private Sub GetAllHandler(ByVal result As ReturnResult(Of IList(Of IProgramInfo)))
			m_WasNotifyCalled = True
			m_DataReturnedFromGetAllCall = result.Result

			Assert.IsNull(result.Error, "Got an exception.  '{0}'", result.Error)
			Assert.IsNotNull(result.Result, "Did not get a result.")
		End Sub

		<TestMethod>
		Public Sub GetById()
			Dim expectedId As String = "RSS_FEED.ITS_ALL_POLITICS"
			RepositoryInstance.GetById(New ReturnResult(Of IProgramInfo)(AddressOf GetByIdHandler), expectedId)

			Assert.IsTrue(m_WasNotifyCalled, "Notify wasn't called.")

			Assert.IsNotNull(m_DataReturnedFromGetByIdCall, "Got null data from call.")

			Assert.AreEqual(Of String)(expectedId, m_DataReturnedFromGetByIdCall.Id, "Id value was wrong.")
			Assert.AreEqual(Of String)("It's All Politics", m_DataReturnedFromGetByIdCall.Name, "Name was wrong.")
		End Sub

		Private Sub GetByIdHandler(ByVal result As ReturnResult(Of IProgramInfo))
			m_WasNotifyCalled = True
			m_DataReturnedFromGetByIdCall = result.Result

			Assert.IsNull(result.Error, "Got an exception.  '{0}'", result.Error)
			Assert.IsNotNull(result.Result, "Did not get a result.")
		End Sub

		<TestMethod>
		Public Sub AssertProgramInforationFileDoesNotExistInStorageAtStartOfTest()
			Assert.IsFalse(UnitTestUtility.FileExists(DataAccessConstants.FilenameProgramInformation), "Program information file should not exist.")
		End Sub

		<TestMethod>
		Public Sub SaveCurrentProgramId()
			RepositoryInstance.SaveCurrentProgramId("asdf")

			Assert.IsTrue(UnitTestUtility.FileExists(DataAccessConstants.FilenameProgramInformation), "Program information file does not exist.")
		End Sub

		<TestMethod>
		Public Sub SaveCurrentProgramIdAcceptsNull()
			RepositoryInstance.SaveCurrentProgramId(Nothing)

			Assert.IsTrue(UnitTestUtility.FileExists(DataAccessConstants.FilenameProgramInformation), "Program information file does not exist.")
		End Sub

		<TestMethod>
		Public Sub GetCurrentProgramId()
			Dim expected As String = "asdf"

			RepositoryInstance.SaveCurrentProgramId(expected)

			Dim actual = RepositoryInstance.GetCurrentProgramId

			Assert.AreEqual(Of String)(expected, actual, "CurrentProgramId did not match.")
		End Sub

		<TestMethod>
		Public Sub GetCurrentProgramIdWhenValueIsEmptyStringReturnsNull()
            Dim expected As String = String.Empty

			RepositoryInstance.SaveCurrentProgramId(String.Empty)

			Dim actual = RepositoryInstance.GetCurrentProgramId

			Assert.AreEqual(Of String)(expected, actual, "CurrentProgramId did not match.")
		End Sub

		<TestMethod>
		Public Sub GetCurrentProgramIdWhenValueIsNullStringReturnsNull()
            Dim expected As String = String.Empty

			RepositoryInstance.SaveCurrentProgramId(Nothing)

			Dim actual = RepositoryInstance.GetCurrentProgramId

			Assert.AreEqual(Of String)(expected, actual, "CurrentProgramId did not match.")
		End Sub
	End Class
End Namespace
