Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Microsoft.Silverlight.Testing
Imports Benday.Npr.Phone.Common.Repositories
Imports Benday.Npr.Phone.Common.Models

Namespace Benday.Npr.Phone.UnitTests.DataAccess
	<TestClass>
	Public Class StoryXmlAdapterFixture
		Inherits UnitTestBase

		Public Sub New()
		End Sub

		<TestMethod>
		Public Sub StoriesXmlAdapter_ParseAllThingsConsideredStories()
			Dim fromValue = TestDataUtility.AllThingsConsideredStories

			Dim adapter = ObjectFactoryInstance.Create(Of IStoryXmlAdapter)()
			Dim toValue = ObjectFactoryInstance.Create(Of IStoryList)()

			adapter.Adapt(fromValue, toValue)

			Assert.AreEqual(Of String)("NPR: All Things Considered", toValue.Title, "Title was wrong.")

			Assert.AreEqual(Of String)("Every weekday, All Things Considered hosts Robert Siegel, Michele Norris and Melissa Block present the program's trademark mix of news, interviews, commentaries, reviews, and offbeat features.", toValue.Teaser, "Teaser was wrong.")
			Assert.AreEqual(Of String)("A trademark mix of news, interviews, commentaries, reviews, and offbeat features.", toValue.TeaserMini, "TeaserMini was wrong.")

			Assert.AreEqual(Of String)("http://www.npr.org/programs/all-things-considered/?ft=3&f=2", toValue.UrlToHtml, "UrlToHtml")

			Assert.AreEqual(Of String)("http://api.npr.org/query/query?id=2&apiKey=MDA2MzU3ODU0MDEyODg2MTQyOTk1OWQ4NQ001", toValue.UrlToApi, "UrlToApi")

			Assert.IsNotNull(toValue.Stories, "Stories was null.")
			Assert.AreNotEqual(Of Integer)(0, toValue.Stories.Count, "Story count was wrong.")

			For Each story In toValue.Stories
				Assert.IsNotNull(story, "Story was null.")
				Assert.AreNotEqual(Of String)(String.Empty, story.Title, "Title should not be empty.")
				UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(story.Title, "story.Title")

				For Each paragraph In story.Paragraphs
					UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(paragraph.Text, "paragraph.Text")
				Next paragraph
			Next story
		End Sub

		<TestMethod, Tag("UndisplayableHtml")>
		Public Sub StoriesXmlAdapter_ParseTellMeMoreStories_RemoveUndisplayableHtml()
			Dim fromValue = TestDataUtility.TellMeMoreStories

			Dim adapter = ObjectFactoryInstance.Create(Of IStoryXmlAdapter)()
			Dim toValue = ObjectFactoryInstance.Create(Of IStoryList)()

			adapter.Adapt(fromValue, toValue)

			UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(toValue.Title, "toValue.Title")
			UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(toValue.Teaser, "toValue.Teaser")
			UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(toValue.TeaserMini, "toValue.TeaserMini")

			Assert.IsNotNull(toValue.Stories, "Stories was null.")
			Assert.AreNotEqual(Of Integer)(0, toValue.Stories.Count, "Story count was wrong.")

			For Each story In toValue.Stories
				Assert.IsNotNull(story, "Story was null.")
				Assert.AreNotEqual(Of String)(String.Empty, story.Title, "Title should not be empty.")
				UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(story.Title, "story.Title")

				For Each paragraph In story.Paragraphs
					UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(paragraph.Text, "paragraph.Text")
				Next paragraph
			Next story
		End Sub

		<TestMethod>
		Public Sub StoriesXmlAdapter_ParseStory()
			Dim fromValue = TestDataUtility.IndividualStory

			Dim adapter = ObjectFactoryInstance.Create(Of IStoryXmlAdapter)()
			Dim toValue = ObjectFactoryInstance.Create(Of IStory)()

			adapter.Adapt(fromValue, toValue)

			Assert.AreEqual(Of String)("131216486", toValue.Id, "Id")
			Assert.AreEqual(Of String)("http://www.npr.org/2010/11/10/131216486/in-israel-when-is-a-jew-not-jewish-enough?ft=3&f=2", toValue.UrlToHtml, "UrlToHtml")
			Assert.AreEqual(Of String)("http://api.npr.org/query?id=131216486&apiKey=MDA2MzU3ODU0MDEyODg2MTQyOTk1OWQ4NQ001", toValue.UrlToApi, "UrlToApi")
			Assert.AreEqual(Of String)("http://n.pr/9k87A3", toValue.UrlShort, "UrlShort")

			Assert.AreEqual(Of String)("In Israel, When Is A Jew Not Jewish Enough?", toValue.Title, "Title")
			Assert.AreEqual(Of String)("The question of Jewish identity is troubling the Israel Defense Forces -- and it's affecting a number of American Jews. The debate highlights the fault lines between different strands of Judaism and the growing influence of ultra-Orthodox Jews in Israel.", toValue.Teaser, "Teaser")
			Assert.AreEqual(Of String)("The debate over Jewish identity highlights the growing influence of ultra-Orthodox Jews in Israel.", toValue.TeaserMini, "TeaserMini")

			Assert.AreEqual(Of String)("Middle East", toValue.Slug, "Slug")

			Assert.AreEqual(Of Date)(New Date(2010, 11, 10, 15, 0, 0), toValue.StoryDate, "StoryDate")
			Assert.AreEqual(Of Date)(New Date(2010, 11, 10, 17, 21, 0), toValue.PublishDate, "PublishDate")

			Assert.AreEqual(Of String)("All Things Considered", toValue.ShowName, "ShowName")
			Assert.AreEqual(Of Date)(New Date(2010, 11, 10, 15, 0, 0), toValue.ShowDate, "ShowDate")

			Assert.AreEqual(Of String)("National Public Radio", toValue.OrganizationName, "OrganizationName")
			Assert.AreEqual(Of String)("NPR", toValue.OrganizationAbbreviation, "OrganizationAbbreviation")
			Assert.AreEqual(Of String)("http://www.npr.org/", toValue.OrganizationUrl, "OrganizationUrl")

			Assert.AreEqual(Of Integer)(283, toValue.Duration, "StoryDuration")

			Assert.AreEqual(Of String)("http://api.npr.org/m3u/1131216499-490434.m3u?orgId=1&topicId=1009&ft=3&f=2", toValue.UrlToMp3, "UrlToMp3")
			Assert.AreEqual(Of String)("http://www.npr.org/templates/dmg/dmg_wmref_em.php?id=131216499&type=1&mtype=WM&orgId=1&topicId=1009&ft=3&f=2", toValue.UrlToWindowsMedia, "UrlToWindowsMedia")

			Assert.IsNotNull(toValue.Paragraphs, "Paragraphs collection was null.")
			Assert.AreEqual(Of Integer)(28, toValue.Paragraphs.Count, "Paragraph count was wrong.")

			For Each paragraph In toValue.Paragraphs
				Assert.IsNotNull(paragraph, "Paragraph was null.")
				Assert.IsNotNull(paragraph.Text, "Paragraph.Text was null.")
				Assert.AreNotEqual(Of String)(String.Empty, paragraph.Text, "paragraph.Text should not be empty.")
				UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(paragraph.Text, "paragraph.Text")
			Next paragraph

			Assert.AreEqual(Of String)("Lourdes Garcia-Navarro", toValue.Author, "Author")
		End Sub
	End Class
End Namespace
