Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports System.IO.IsolatedStorage

Namespace Benday.Npr.Phone.UnitTests.DataAccess
	Public Class UnitTestUtility
		Public Sub New()
		End Sub

		Public Shared Sub AssertNotNullOrEmpty(ByVal actual As String, ByVal fieldName As String)
			Assert.AreNotEqual(Of String)(Nothing, actual, "'{0}' should not be null.", fieldName)
			Assert.AreNotEqual(Of String)(String.Empty, actual, "'{0}' should not be an empty string.", fieldName)
		End Sub

		Public Shared Sub AssertStringDoesNotContainUndisplayableHtmlFormatting(ByVal checkThisString As String)
			Assert.IsFalse(StringContainsUndisplayableHtmlFormatting(checkThisString), "Should not contain any undisplayable html formatting.  ('{0}')", checkThisString)
		End Sub

		Public Shared Sub AssertStringDoesNotContainUndisplayableHtmlFormatting(ByVal checkThisString As String, ByVal message As String)
			Assert.IsFalse(StringContainsUndisplayableHtmlFormatting(checkThisString), "{1} should not contain any undisplayable html formatting.  ('{0}')", checkThisString, message)
		End Sub

		Public Shared Function StringContainsUndisplayableHtmlFormatting(ByVal checkThisString As String) As Boolean
			Assert.IsNotNull(checkThisString, "String was null.")

			If checkThisString.Contains("&lt;em&gt;") = True OrElse checkThisString.Contains("&lt;/em&gt;") = True OrElse checkThisString.Contains("<em>") = True OrElse checkThisString.Contains("</em>") = True OrElse checkThisString.Contains("<strong>") = True OrElse checkThisString.Contains("</strong>") = True Then
				Return True
			Else
				Return False
			End If
		End Function

		Public Shared Sub WipeProgramDataInIsolatedStorage()
			Using userIsolatedStorage = IsolatedStorageFile.GetUserStoreForApplication
				Dim files = userIsolatedStorage.GetFileNames

				For Each file In files
					userIsolatedStorage.DeleteFile(file)
				Next file
			End Using
		End Sub

		Public Shared Function FileExists(ByVal filename As String) As Boolean
			Using userIsolatedStorage = IsolatedStorageFile.GetUserStoreForApplication
				Dim files = userIsolatedStorage.GetFileNames

				If files Is Nothing Then
					Return False
				Else
                    Dim match = (
                        From temp In files
                        Where temp = filename
                        Select temp).FirstOrDefault

					If match Is Nothing Then
						Return False
					Else
						Return True
					End If
				End If
			End Using
		End Function
	End Class
End Namespace
