Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Benday.Npr.Phone.Presentation.Interfaces
Imports Benday.Npr.Phone.Presentation
Imports Benday.Npr.Phone.UnitTests.DataAccess
Imports Benday.Npr.Phone.Common.Repositories

Namespace Benday.Npr.Phone.UnitTests.Presentation
	<TestClass>
	Public Class ProgramsViewModelFixture
		Inherits UnitTestBase

		Protected Overrides Sub OnTestInitialize()
			UnitTestUtility.WipeProgramDataInIsolatedStorage()
			m_ViewModelInstance = Nothing
			MyBase.OnTestInitialize()
		End Sub

		Private m_ViewModelInstance As IProgramsViewModel
		Public ReadOnly Property ViewModelInstance As IProgramsViewModel
			Get
				If m_ViewModelInstance Is Nothing Then
					m_ViewModelInstance = ObjectFactoryInstance.Create(Of IProgramsViewModel)()
				End If

				Return m_ViewModelInstance
			End Get
		End Property

		<TestMethod>
		Public Sub AllProgramsViewModel_Load()
			ViewModelInstance.Load()

			Assert.IsNotNull(ViewModelInstance.Programs, "Programs was null.")

			Assert.AreEqual(Of Integer)(16, ViewModelInstance.Programs.Items.Count, "Program count was wrong.")

			AssertProgram(ViewModelInstance, 0, "NPR Hourly News Summary", "RSS_FEED.NPR_HOURLY_NEWS")
			AssertProgram(ViewModelInstance, 1, "All Things Considered", "2")
			AssertProgram(ViewModelInstance, 2, "Fresh Air from WHYY", "13")
			AssertProgram(ViewModelInstance, 3, "It's All Politics", "RSS_FEED.ITS_ALL_POLITICS")
			AssertProgram(ViewModelInstance, 12, "Tell Me More", "46")
		End Sub

		<TestMethod>
		Public Sub ProgramsViewModel_SelectItem_TriggersUpdatedEvent_UpdatesSelectedItemId()
			ViewModelInstance.Load()

			Assert.IsNotNull(ViewModelInstance.Programs, "Programs was null.")

			Assert.AreEqual(Of Integer)(16, ViewModelInstance.Programs.Items.Count, "Program count was wrong.")

			Assert.IsNull(ViewModelInstance.Programs.SelectedItem, "Nothing should be selected yet.")

			Assert.AreEqual(Of String)(Nothing, ViewModelInstance.CurrentProgramId, "Current program id value was wrong before selection.")

			Dim expectedSelection = ViewModelInstance.Programs.Items(0)

			Dim tester = New ProgramsViewModelEventTester(ViewModelInstance)

			expectedSelection.IsSelected = True

			Assert.IsTrue(tester.GotSelectedEvent, "Should have gotten selection changed event.")

			Assert.AreEqual(Of String)(expectedSelection.Id, ViewModelInstance.CurrentProgramId, "Current program id value was wrong.")
		End Sub

		<TestMethod>
		Public Sub ProgramsViewModel_SelectItem_SavesCurrentProgramIdToIsolatedStorage()
			ViewModelInstance.Load()

			Assert.IsNotNull(ViewModelInstance.Programs, "Programs was null.")

			SelectProgramAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 0)
			SelectProgramAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 1)
		End Sub

		<TestMethod>
		Public Sub ProgramsViewModel_WhenLoadMethodIsCalledCurrentProgramIdIsLoadedOnCall()
			ViewModelInstance.Load()

			Assert.IsNull(ViewModelInstance.CurrentProgramId, "CurrentProgramId should be null.")

			SelectProgramAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 0)

			Dim expectedCurrentProgramId = ViewModelInstance.CurrentProgramId

			m_ViewModelInstance = Nothing

			ViewModelInstance.Load()

			Assert.AreEqual(Of String)(expectedCurrentProgramId, ViewModelInstance.CurrentProgramId, "CurrentProgramId did not match expected value after call to Load.")
		End Sub

		Private Sub SelectProgramAndAssertIdIsSavedToIsolatedStorage(ByVal viewModel As IProgramsViewModel, ByVal index As Integer)
			Dim selectThis = viewModel.Programs.Items(index)

			selectThis.IsSelected = True

			Dim repository = ObjectFactoryInstance.Create(Of IProgramRepository)()

			Dim currentProgramId = repository.GetCurrentProgramId

			Assert.AreEqual(Of String)(selectThis.Id, currentProgramId, "Program Id did not match what was stored.")
		End Sub

		Private Sub AssertProgram(ByVal instance As IProgramsViewModel, ByVal programIndex As Integer, ByVal expectedTitle As String, ByVal expectedId As String)
			Dim actual = instance.Programs.Items(programIndex)

			Assert.AreEqual(Of String)(expectedTitle, actual.Text, "Text")
			Assert.AreEqual(Of String)(expectedId, actual.Id, "Id")
		End Sub
	End Class
End Namespace
