Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Benday.Npr.Phone.Presentation.Interfaces
Imports Benday.Npr.Phone.UnitTests.DataAccess
Imports Benday.Npr.Phone.Common.Repositories
Imports Benday.Npr.Phone.DataAccess
Imports Benday.Npr.Phone.Presentation

Namespace Benday.Npr.Phone.UnitTests.Presentation
	<TestClass>
	Public Class StoryViewModelFixture
		Inherits UnitTestBase

		Protected Overrides Sub OnTestInitialize()
			UnitTestUtility.WipeProgramDataInIsolatedStorage()
			m_ViewModelInstance = Nothing
			m_RepositoryInstance = Nothing
			m_ProgramRepositoryInstance = Nothing
			MyBase.OnTestInitialize()
		End Sub

		Private m_ViewModelInstance As IStoryViewModel
		Public ReadOnly Property ViewModelInstance As IStoryViewModel
			Get
				If m_ViewModelInstance Is Nothing Then
					m_ViewModelInstance = New StoryViewModel(ProgramRepositoryInstance, RepositoryInstance, ObjectFactoryInstance)
				End If

				Return m_ViewModelInstance
			End Get
		End Property

		Private m_RepositoryInstance As StoryRepository
		Public ReadOnly Property RepositoryInstance As StoryRepository
			Get
				If m_RepositoryInstance Is Nothing Then
					m_RepositoryInstance = TryCast(ObjectFactoryInstance.Create(Of IStoryRepository)(), StoryRepository)

					m_RepositoryInstance.TestModeConfiguration = New StoryRepositoryTestModeConfiguration With {.TestData = UnitTestResources.AllThingsConsideredStories}
				End If

				Return m_RepositoryInstance
			End Get
		End Property

		Private m_ProgramRepositoryInstance As IProgramRepository
		Public ReadOnly Property ProgramRepositoryInstance As IProgramRepository
			Get
				If m_ProgramRepositoryInstance Is Nothing Then
					m_ProgramRepositoryInstance = New ProgramRepositoryFake
				End If

				Return m_ProgramRepositoryInstance
			End Get
		End Property

		<TestMethod>
		Public Sub CreateInstanceOfStoryViewModelViaObjectFactory()
			Dim item = ObjectFactoryInstance.Create(Of IStoryViewModel)()

			Assert.IsNotNull(item)
		End Sub

		<TestMethod>
		Public Sub LoadFromCacheForCurrentStoryId()
			' force the repository to write the test data to cache
			RepositoryInstance.RefreshStories("2")
			Dim expectedStoryId As String = "131216486"
			Dim expectedStoryTitle As String = "In Israel, When Is A Jew Not Jewish Enough?"

			RepositoryInstance.SaveCurrentStoryId(expectedStoryId)

			ViewModelInstance.Load()

			UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramId, "CurrentProgramId")

			Assert.AreEqual(Of String)(expectedStoryId, ViewModelInstance.Id, "Story Id was wrong.")
			Assert.AreEqual(Of String)(expectedStoryTitle, ViewModelInstance.Title, "Story title was wrong.")
		End Sub
	End Class
End Namespace
