Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Benday.Npr.Phone.Common

Namespace Benday.Npr.Phone.UnitTests
	<TestClass>
	Public Class ReturnResultFixture
		Inherits UnitTestBase

		Public Sub New()
		End Sub

		<TestMethod>
		Public Sub ReturnResult_Notify_Result()
			Dim gotCall As Boolean = False

			Dim expectedResultValue As String = "expected result"

			Dim instance = New ReturnResult(Of String)(New Action(Of ReturnResult(Of String))(Sub(arg)
				gotCall = True
				Assert.IsNull(arg.Error, "Error should be null.")
				Assert.AreEqual(Of String)(expectedResultValue, arg.Result, "Result was wrong.")
				Assert.IsNull(arg.ParentCall, "ParentCall should be null.")
			End Sub))

			instance.Notify(expectedResultValue)

			Assert.IsTrue(gotCall, "Did not receive call.")
		End Sub

		<TestMethod>
		Public Sub ReturnResult_Notify_Error()
			Dim gotCall As Boolean = False

			Dim expectedErrorValue As New ArgumentException("BOOM!")

			Dim instance = New ReturnResult(Of String)(New Action(Of ReturnResult(Of String))(Sub(arg)
				gotCall = True
				Assert.IsNull(arg.Result, "Result should be null.")
				Assert.AreSame(expectedErrorValue, arg.Error, "Error instance was wrong.")
				Assert.IsNull(arg.ParentCall, "ParentCall should be null.")
			End Sub))

			instance.Notify(expectedErrorValue)

			Assert.IsTrue(gotCall, "Did not receive call.")
		End Sub

		<TestMethod>
		Public Sub ReturnResult_NotifyParent_Success()
			Dim gotCall As Boolean = False
			Dim gotParentCall As Boolean = False

			Dim expectedResultValue As String = "expected result"

			Dim parentCall = New ReturnResult(Of String)(New Action(Of ReturnResult(Of String))(Sub(arg)
				gotParentCall = True
				Assert.IsNull(arg.Error, "Error should be null.")
				Assert.AreEqual(Of String)(expectedResultValue, arg.Result, "Result was wrong.")
				Assert.IsNull(arg.ParentCall, "ParentCall should be null.")
			End Sub))

			Dim instance = New ReturnResult(Of String)(New Action(Of ReturnResult(Of String))(Sub(arg)
				gotCall = True
				Assert.IsNull(arg.Error, "Error should be null.")
				Assert.AreEqual(Of String)(expectedResultValue, arg.Result, "Result was wrong.")
				Assert.IsNotNull(arg.ParentCall, "ParentCall should not be null.")
			End Sub), parentCall)

			instance.Notify(expectedResultValue)

			' forward the same result to the parent call
			instance.NotifyParentCall()

			Assert.IsTrue(gotCall, "Did not receive call.")
			Assert.IsTrue(gotParentCall, "Did not receive parent call.")
		End Sub

		<TestMethod>
		Public Sub ReturnResult_NotifyParent_Error()
			Dim gotCall As Boolean = False
			Dim gotParentCall As Boolean = False

			Dim expectedErrorValue As New ArgumentException("BOOM!")

			Dim parentCall = New ReturnResult(Of String)(New Action(Of ReturnResult(Of String))(Sub(arg)
				gotParentCall = True
				Assert.IsNull(arg.Result, "Result should be null.")
				Assert.AreSame(expectedErrorValue, arg.Error, "Error instance was wrong.")
				Assert.IsNull(arg.ParentCall, "ParentCall should be null.")
			End Sub))

			Dim instance = New ReturnResult(Of String)(New Action(Of ReturnResult(Of String))(Sub(arg)
				gotCall = True
				Assert.IsNull(arg.Result, "Result should be null.")
				Assert.AreSame(expectedErrorValue, arg.Error, "Error instance was wrong.")
				Assert.IsNotNull(arg.ParentCall, "ParentCall should be null.")
			End Sub), parentCall)

			instance.Notify(expectedErrorValue)

			' forward the same result to the parent call
			instance.NotifyParentCall()

			Assert.IsTrue(gotCall, "Did not receive call.")
			Assert.IsTrue(gotParentCall, "Did not receive parent call.")
		End Sub
	End Class
End Namespace
