Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Tasks
Imports Benday.Npr.Phone.Presentation
Imports Benday.Npr.Phone.Presentation.Interfaces
Imports Microsoft.Phone.BackgroundAudio

Namespace Benday.Npr.Phone
	Partial Public Class MainPage
		Inherits PhoneApplicationPage

		' Constructor
		Public Sub New()
			InitializeComponent()

			AddHandler Loaded, AddressOf MainPage_Loaded

			PopulateVersionNumber()
		End Sub

		Private Sub PopulateVersionNumber()
			m_textVersion.Text = "v2.0.0"
		End Sub

		Private m_ViewModel As IProgramsViewModel

		Private Sub MainPage_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			m_ViewModel = ViewModelLocator.Instance.Programs

			Me.DataContext = m_ViewModel

			m_ViewModel.Load()

			AddHandler m_ViewModel.Programs.OnItemSelected, AddressOf Programs_OnItemSelected

			CheckIfPlayingAudioAndNavigateIfRequired()
		End Sub

		Private Sub CheckIfPlayingAudioAndNavigateIfRequired()
			If ViewModelLocator.Instance.HasHandledNavigateFromPlaybackAgent = True Then
				Exit Sub
			End If

			If BackgroundAudioPlayer.Instance IsNot Nothing AndAlso BackgroundAudioPlayer.Instance.Track IsNot Nothing Then
				Dim track = BackgroundAudioPlayer.Instance.Track

				If String.IsNullOrWhiteSpace(track.Tag) = False AndAlso track.Tag.StartsWith("nprlistener:") = True Then
					Dim tag As String = track.Tag

					tag = tag.Replace("nprlistener:", String.Empty)

					Dim tokens = tag.Split(New String() { ":" }, StringSplitOptions.None)

					If tokens.Length = 2 Then
						Dim programId As String = tokens(0)
						Dim storyId As String = tokens(1)

						ViewModelLocator.Instance.Programs.Programs.SelectByValue(programId)
					End If
				End If
			End If
		End Sub

		Private Sub Programs_OnItemSelected(ByVal sender As Object, ByVal e As EventArgs)
			NavigateToProgram()
		End Sub

		Private Sub NavigateToProgram()
			If m_ViewModel IsNot Nothing AndAlso m_ViewModel.Programs.SelectedItem IsNot Nothing Then
				NavigationService.Navigate(New Uri("/StoryListPage.xaml", UriKind.Relative))

                '   If m_ViewModel.Programs.SelectedItem.Id = "RSS_FEED.NPR_HOURLY_NEWS" Then
                '          NavigationService.Navigate(New Uri("/HourlyNewsPage.xaml", UriKind.Relative))
                '   Else
                '          NavigationService.Navigate(New Uri("/StoryListPage.xaml", UriKind.Relative))

            End If
        End Sub

		Private Sub HyperlinkButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			Dim senderAsHyperLink As HyperlinkButton = TryCast(sender, HyperlinkButton)

			If senderAsHyperLink IsNot Nothing Then
				Dim task = New WebBrowserTask
				task.Uri = New Uri(TryCast(senderAsHyperLink.Tag, String))

				task.Show()
			End If
		End Sub
	End Class
End Namespace
