Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports Microsoft.Phone.Controls
Imports Benday.Npr.Phone.Presentation.Interfaces
Imports Benday.Npr.Phone.Presentation
Imports Microsoft.Phone.BackgroundAudio

Namespace Benday.Npr.Phone
	Partial Public Class StoryListPage
		Inherits PhoneApplicationPage

		Public Sub New()
			InitializeComponent()
		End Sub

		Private m_ViewModel As IStoryListViewModel

		Private Sub PhoneApplicationPage_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			m_ViewModel = ViewModelLocator.Instance.StoryList

			Me.DataContext = m_ViewModel

			m_ViewModel.Load()

			AddHandler m_ViewModel.Stories.OnItemSelected, AddressOf Stories_OnItemSelected

			CheckIfPlayingAudioAndNavigateIfRequired()

			m_WatcherNavigateToHome = New NavigateToHomeApplicationBarWatcher(TryCast(ApplicationBar.Buttons(0), Microsoft.Phone.Shell.ApplicationBarIconButton))

			m_WatcherRefresh = New RefreshStoriesApplicationBarWatcher(TryCast(ApplicationBar.Buttons(1), Microsoft.Phone.Shell.ApplicationBarIconButton))
		End Sub

		Private m_WatcherNavigateToHome As NavigateToHomeApplicationBarWatcher
		Private m_WatcherRefresh As RefreshStoriesApplicationBarWatcher

		Private Sub CheckIfPlayingAudioAndNavigateIfRequired()
			If ViewModelLocator.Instance.HasHandledNavigateFromPlaybackAgent = True Then
				Exit Sub
			End If

			If BackgroundAudioPlayer.Instance IsNot Nothing AndAlso BackgroundAudioPlayer.Instance.Track IsNot Nothing Then
				Dim track = BackgroundAudioPlayer.Instance.Track

				If String.IsNullOrWhiteSpace(track.Tag) = False AndAlso track.Tag.StartsWith("nprlistener:") = True Then
					Dim tag As String = track.Tag

					tag = tag.Replace("nprlistener:", String.Empty)

					Dim tokens = tag.Split(New String() { ":" }, StringSplitOptions.None)

					If tokens.Length = 2 Then
						Dim programId As String = tokens(0)
						Dim storyId As String = tokens(1)

						ViewModelLocator.Instance.HasHandledNavigateFromPlaybackAgent = True

						ViewModelLocator.Instance.StoryList.Stories.SelectByValue(storyId)
					End If
				End If
			End If
		End Sub

		Private Sub m_HomeButton_Click(ByVal sender As Object, ByVal e As EventArgs)
			NavigationService.Navigate(New Uri("/MainPage.xaml", UriKind.Relative))
		End Sub

		Private Sub m_RefreshButton_Click(ByVal sender As Object, ByVal e As EventArgs)
			m_ViewModel.Refresh()
		End Sub

		Private Sub Stories_OnItemSelected(ByVal sender As Object, ByVal e As EventArgs)
			NavigateToStory()
		End Sub

		Private Sub NavigateToStory()
			If m_ViewModel IsNot Nothing AndAlso m_ViewModel.Stories.SelectedItem IsNot Nothing Then
				NavigationService.Navigate(New Uri("/StoryDetailsPage.xaml", UriKind.Relative))
			End If
		End Sub
	End Class
End Namespace
